#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/10/31 13:21
# @Author: Jtyoui@qq.com
from jtyoui.data import reader_conf
import time
import configparser
import os
import datetime
import re

NOW_TIME = time.localtime(time.time())


class ParseTime:

    def __init__(self, data, current_date=None, date_format='%Y-%m-%d %H:%M:%S', **kwargs):
        """初始化数据日期
        :param data:当前数据
        :param current_date:当前日期
        :param date_format:日期解析格式
        :param kwargs:其他参数
        map_path :解析日期的映射表
        re_path :匹配日期的正则表
        """
        # 加载日期的映射表和匹配日期的正则表
        self.map, self.re = None, None
        self.load_config(**kwargs)

        self.data = data + '\033'
        self._decide_ten()
        # 定义当前日期
        self.localtime = current_date if current_date else time.strftime(date_format)

        self.format = date_format

        # 将当前日期标准化
        local = time.strptime(self.localtime, self.format)

        # 初始化当前的年月日基本信息
        self.now_year = local.tm_year
        self.now_mon = local.tm_mon
        self.now_day = local.tm_mday
        self.now_week = local.tm_wday + 1
        if current_date:
            self.now_hour = local.tm_hour
            self.now_minute = local.tm_min
            self.now_second = local.tm_sec
        else:
            self.now_hour = 0
            self.now_minute = 0
            self.now_second = 0

        self.reduction = True  # 启动还原时分秒

    def load_config(self, map_path=None, re_path=None):
        """自定义日期解析映射表和匹配日期的正则表
        :param map_path: 解析日期的映射表
        :param re_path: 匹配日期的正则表
        """
        path = os.path.dirname(__file__)
        self.map = configparser.ConfigParser()
        if map_path:
            self.map.read(map_path, encoding='UTF-8')
        else:
            self.map.read(path + os.sep + 'map.ini', encoding='UTF-8')
        if re_path:
            self.re = reader_conf(re_path)
        else:
            self.re = reader_conf(path + os.sep + 're.cfg')

    def change_time(self, day=0, hour=0, minute=0, week=0, second=0):
        """增加天数来修改时间"""
        add_time = datetime.timedelta(days=day, hours=hour, minutes=minute, weeks=week, seconds=second)
        if self.reduction:
            change = F'{self.now_year}-{self.now_mon}-{self.now_day} 00:00:00'  # 时分秒还原到0
            self.reduction = False
        else:
            change = self.str_time()
        add = datetime.datetime.strptime(change, self.format) + add_time
        self.now_year = add.year
        self.now_mon = add.month
        self.now_day = add.day
        self.now_hour = add.hour
        self.now_minute = add.minute
        self.now_second = add.second
        self.now_week = add.isoweekday()

    def _analysis(self, name):
        """分析数据获取年月日周时分秒中的有效信息
        :param name: 年月日时周分秒调用名字
        :return: 改变的值
        """
        re_year = '|'.join(self.re['re_' + name])
        ls = re.search(re_year, self.data)
        if ls is not None:
            message = ls.group()
            add_year = self.map['add_' + name]
            value = add_year.get(message, 0)
        else:
            value = 0
        return int(value)

    def standard_time(self):
        """标准时间化"""
        return time.strptime(self.str_time(), self.format)

    def str_time(self):
        """字符串时间"""
        return self.__str__()

    def day(self):
        """查询天数"""
        value = self._analysis('day')
        if value == 0:
            re_year = '|'.join(self.re['re_day'])
            change_day = re.search(re_year, self.data)
            if change_day:
                message = change_day.group()
                if message[:-1].isdigit():
                    self.now_day = message[:-1]
                elif '后' in message:
                    value = int(message[:message.find('天')])
                elif '前' in message:
                    value = -int(message[:message.find('天')])
        self.change_time(day=value)

    def month(self):
        """查询月份"""
        value = self._analysis('month')
        if value == 0:
            re_month = '|'.join(self.re['re_month'])
            change_month = re.search(re_month, self.data)
            if change_month:
                message = change_month.group()
                if '个月' in message:
                    point = message[:message.find('个')]
                else:
                    point = message[:message.find('月')]
                if point.isdigit():
                    point = int(point)
                    if '底' in message:
                        self.now_mon = point + 1
                        self.now_day = 1
                        self.change_time(day=-1)
                    elif '除' in message:
                        self.now_mon = point
                        self.now_day = 1
                    elif '后' in message:
                        value = point
                    elif '前' in message:
                        value = -point
                    else:
                        self.now_mon = point
        if value:
            mon = self.now_mon + value
            if mon == 0:
                self.now_year -= 1
                mon = 12
            if mon > 12:
                over_12_mon, mod_12 = divmod(mon, 12)
                self.now_year += over_12_mon
                self.now_mon = mod_12
            else:
                self.now_mon = mon

    def year(self):
        """查询年份"""
        value = self._analysis('year')
        if value == 0:
            re_year = '|'.join(self.re['re_year'])
            change_year = re.search(re_year, self.data)
            if change_year:
                message = change_year.group()
                if message[:-1].isdigit():
                    self.now_year = int(message[:-1])
        else:
            self.now_year += value

    def week(self):
        """查找第几个周"""
        value = self._analysis('week')
        self.change_time(week=value)

    def what_week(self):
        """查找当前周中的第几个星期"""
        ds = self.map['chinese_number']
        keys = r'星期(\d+|' + '|'.join(ds.keys())[4:-11] + '天|日)'
        match = re.search(keys, self.data)
        if match is not None:
            value = match.group()
            point = value[2:]
            if point.isdigit():
                w = int(point)
            elif ds.get(point, None) is not None:
                w = int(ds[point])
            elif point in '天日':
                w = 7
            else:
                w = self.now_week  # 暂未设置
            differ = w - self.now_week
            self.change_time(day=differ)

    def hour(self):
        """查找当前的小时"""
        re_hour = '(' + '|'.join(self.re['re_hour']) + ')' + r'\d+点'
        match = re.search(re_hour, self.data)
        if match is not None:
            value = match.group()
            add_hour = int(value[2:-1])
            point = value[:2]
            sc = int(self.map['add_hour'].get(point, 0))
            if add_hour < sc:
                if '凌晨' in value and add_hour == 12:
                    add_hour = sc  # 凌晨12点等于凌晨0点
                else:
                    add_hour += sc
            self.change_time(hour=add_hour)
        else:
            self.reduction = False

    def minute(self):
        """查找当前的分钟"""
        re_minute = '|'.join(self.re['re_minute'])
        match = re.search(re_minute, self.data)
        if match is not None:
            value = match.group()
            if '点半' in value:
                add_min = 30
            elif '分' in value:
                add_min = int(value[:-1])
            else:
                add_min = 0  # 其他情况
            self.change_time(minute=add_min)

    def second(self):
        """查找当前的秒钟"""
        re_second = '|'.join(self.re['re_second'])
        match = re.search(re_second, self.data)
        if match is not None:
            value = match.group()
            if '秒' in value:
                add_second = int(value[:-1])
                self.change_time(second=add_second)

    def parse(self):
        """开始解析，返回解析后的标准时间"""
        self.second()
        self.minute()
        self.hour()
        self.week()
        self.what_week()
        self.day()
        self.month()
        self.year()
        return self

    def find_times(self):
        """同StringTime.find_times中的一样"""
        return [self.parse().__str__()]

    def _decide_ten(self):
        """重新映射一些词语"""
        chinese_number = self.map['chinese_number']
        str_ = list()
        for data in self.data:
            if data == '周':
                # 如果又有周又有星期，不改变
                if '星期' in self.data:
                    str_.append(data)
                else:  # 否则将周改为星期
                    str_.append(self.map['chinese_number'].get(data, data))
            elif data != '十' and data in chinese_number:  # 十为特殊符号
                str_.append(self.map['chinese_number'][data])
            else:
                str_.append(data)
        if '十' in str_:
            # 判断十在每个位置上的不同意义
            for index, c in enumerate(str_):
                if c == '十':
                    if str_[index - 1].isdigit() and str_[index + 1].isdigit():  # 比如：二十一实际上十可以取空，变成21
                        str_[index] = ''
                    elif str_[index - 1].isdigit() and (not str_[index + 1].isdigit()):  # 比如：二十实际上十变成0，变成20
                        str_[index] = '0'
                    elif not str_[index - 1].isdigit() and str_[index + 1].isdigit():  # 比如：十三实际上十变成1，变成13
                        str_[index] = '1'
                    else:
                        str_[index] = '10'  # 其余情况十就变成10
        self.data = ''.join(str_[:-1])

    def __add__(self, other):
        """两个时间对象相加"""
        pass

    def __mul__(self, other):
        """两个时间对象相减"""
        pass

    def __str__(self):
        """字符串格式化"""
        return F'{self.now_year}-{self.now_mon}-{self.now_day} ' \
               F'{self.now_hour:0>2}:{self.now_minute:0>2}:{self.now_second:0>2}'


if __name__ == '__main__':
    today = '2019-10-31 16:00:00'
    pt = ParseTime('上上个周星期天下午2点25分钟30秒', today).parse()
    print(pt)  # 2019-10-20 14:25:30

    print('-----------------切换日期------------------')
    st = ParseTime('下周星期一下午2点半开会', today).parse()
    print(st)  # 2019-11-4 14:30:00

    print('----------------多个时间-------------------')
    st = ParseTime('今天下午3点', today).parse()
    print(st)  # 2019-10-31 15:00:00

    print('----------------没有时间-------------------')
    st = ParseTime('我要是不传时间呢？', today).parse()
    print(st)  # 2019-10-31 16:00:00

    print('---------------只有天数--------------------')
    st = ParseTime('明天去哪里？', today).parse()
    print(st)  # 2019-11-1 16:00:00

    print('---------------没有日期或者天数--------------------')
    st = ParseTime('下午2点半开会', today).parse()
    print(st)  # 2019-10-31 14:30:00

    print('---------------*几个月以后--------------------')
    st = ParseTime('下个月1号下午3点', today).parse()
    print(st)  # 2019-11-1 15:00:00

    print('--------------几天之后--------------')
    st = ParseTime('三天之前下午3点', today).parse()
    print(st)  # 2019-10-28 15:00:00

    print('--------------几月底--------------')
    st = ParseTime('明年的2月底之前必须交报告', today).parse()
    print(st)  # 2020-2-28 16:00:00

    print('--------晚上-----------')
    st = ParseTime('3月除', today).parse()
    print(st)  # 2019-3-1 16:00:00

    print('--------下个周几-----------')
    st = ParseTime('下个周2', today).parse()
    print(st)  # 2019-11-5 16:00:00

    print('--------几个月以后的日期--------')
    st = ParseTime('5个月后的明天', today).parse()
    print(st)  # 2020-4-1 16:00:00

    print('------------凌晨或者半夜------------------')
    st = ParseTime('昨天凌晨3点', today).parse()
    print(st)  # 2019-10-31 03:00:00

    print('-------------只说时间-----------------------')
    st = ParseTime('二零零七年八月二十一号下午2点半', today).parse()
    print(st)  # 2007-8-21 14:30:00



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/23 17:15
# @Author: Jtyoui@qq.com
from .Log import get_log_config, set_log_file_config, log_file_config  # 加载日志文件
from .logDecorator import *  # 增加日志的装饰器函数

"""
默认：info日志是1M分割
error日志是每天分割
"""



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/18 9:36
# @Author: Jtyoui@qq.com
from jtyoui import parameter_set_length

"""
余弦距离
几何中，夹角余弦可用来衡量两个向量方向的差异；
机器学习中，借用这一概念来衡量样本向量之间的差异。
余弦相似度用向量空间中两个向量夹角的余弦值作为衡量两个个体间差异的大小。
相比距离度量，余弦相似度更加注重两个向量在方向上的差异，而非距离或长度上
"""


@parameter_set_length
def cosine(coordinate_p, coordinate_q):
    """
    余弦距离又叫余弦角度
    :param coordinate_p:p坐标
    :param coordinate_q:q坐标
    :return:余弦距离
    """
    numerator, denominator = 0, 1  # 分子和父母
    for x, y in zip(coordinate_p, coordinate_q):
        numerator += x * y
    q = [x ** 2 for x in coordinate_q]
    p = [y ** 2 for y in coordinate_p]
    return numerator / pow(sum(q) * sum(p), 0.5)


def cosine_distance(sentence, data):
    import jieba  # 安装 pip install jieba
    ls, ds = [], []
    s = jieba.cut(sentence)
    d = jieba.cut(data)
    for c in set(list(s) + list(d)):
        if len(c) > 0:
            ls.append(sentence.count(c))
            ds.append(data.count(c))
    return cosine(ls, ds)


if __name__ == '__main__':
    print(cosine_distance('我吃饭了', '我正在吃饭'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/12 13:13
# @Author: Jtyoui@qq.com
import re
from jtyoui.error import InconsistentLengthError
import operator


class Tool:
    """自定义工具类"""
    generator = True

    def __init__(self, string):
        self._string = string

    def index_select_string(self, index, select):
        """利用索引的关系来标记字符串"""
        """
        利用索引的关系来找字符串:一般用在深度学习中的标注模型
        :param index: 索引
        :param string: 字符串
        :param select: 索引匹配的正则
        :return: 匹配字符串列表
        """
        ls, string = [], self._string
        if len(index) != len(self._string):
            raise InconsistentLengthError("参数index和参数string长度不一致错误!")
        while True:
            s = re.search(select, index)
            if s:
                ls.append(string[s.start():s.end()])
                index = index[s.start() + 1:]
                string = string[s.start() + 1:]
            else:
                break
        return ls

    @property
    def string(self):
        return self._string

    @string.setter
    def string(self, string):
        self._string = string

    def split(self, re_, flag=0, retain=True):
        """支持正则分割
        :param re_:正则表达式
        :param flag: re.search(re_, self.string, flag), 默认flag=0
        :param retain: 是否要保留正则匹配的字符,默认是保留
        """
        ls_index, ls_word, string, end = [0], [], self._string, 0
        while True:
            match = re.search(re_, string, flag)
            if match:
                ls_index.append(match.start() + end)
                end += match.end()
                string = string[match.end():]
            else:
                break
        if ls_index[-1] != len(self._string):
            ls_index.append(len(self._string))
        if ls_index[1] == 0:
            ls_index.pop(0)
        for index, value in enumerate(ls_index):
            if index == len(ls_index) - 1:
                break
            ls_word.append(self._string[value:ls_index[index + 1]])
        if not retain:
            ls_words = []
            for word in ls_word:
                match = re.search(re_, word, flag)
                if match:
                    w = word[match.end():]
                    if w:
                        ls_words.append(w)
                else:
                    ls_words.append(word)
            ls_word = ls_words
        return ls_word

    def string_select_index(self, ls, start_name, end_name, flag='O', labels=None):
        """将一段文字进行标记返回标记的列表
        :param ls:标记的关键字列表
        :param start_name:开始标记的名称
        :param end_name:连续标记的名称
        :param flag:不在关键字列表中默认标记，默认是O,大写的o
        :param labels:自定义标记
        """
        if not labels:
            labels = [flag for _ in range(len(self._string))]
        for word in ls:
            index = 0
            while True:
                start = self._string.find(word, index)
                if start > -1:
                    end = start + len(word)
                    labels[start] = start_name
                    for i in range(start + 1, end):
                        labels[i] = end_name
                    index = end
                else:
                    break
        return labels

    def select_ls(self, ls_):
        """根据列表里面的元素选取字符串中的元素
        :param ls_:列表元素，比如['张三','李四','王麻子']，string='张三去李四家找东西'
        :return :['张三','李四']
        """
        if self.generator:
            return (name for name in ls_ if self._string.find(name) > 0)
        else:
            return [name for name in ls_ if self._string.find(name) > 0]

    def select_row(self, iterable_, row):
        """选取可迭代对象中的某一列
        :param iterable_:可迭代对象
        :param row:每一列
        """
        g = operator.itemgetter(row)
        if self.generator:
            return (g(i) for i in iterable_)
        else:
            return [g(i) for i in iterable_]

    def search(self, pattern, flags=0):
        """根据正则获取字符串的索引以及值，索引和值都是list类型"""
        r = re.search(pattern, self.string, flags=flags)
        start, end, value = [], [], []
        flag = ''
        while r:
            start_ = r.start() + len(flag)
            end_ = r.end() + len(flag)
            start.append(start_)
            end.append(end_)
            value.append(self.string[start_:end_])
            flag = self.string[:end_]
            string = self.string[end_:]
            r = re.search(pattern, string, flags=flags)

        class A:

            @staticmethod
            def start():
                return start

            @staticmethod
            def end():
                return end

            @staticmethod
            def value():
                return value

        return A()

    def replace(self, pattern, repl, count=0, flags=0):
        """正则替换"""
        return re.sub(pattern, repl, self.string, count=count, flags=flags)


if __name__ == '__main__':
    tool = Tool('我家在贵州省遵义县的一个地方是虾子')
    i_s = tool.index_select_string('01056666600000056', '56+')
    print(i_s)
    tool.string = '我在这里、一、相亲最大的好处是。二、想要什么婚姻。三五、开放型的婚姻是凉鞋。三、'
    t_s = tool.split('[一二三四五六七八九十]+、', retain=False)
    print(t_s)
    tool.string = '我家在贵州省遵义县的一个地方是虾子'
    s_i = tool.string_select_index(ls=['贵州省', '遵义县', '虾子'], start_name='5', end_name='6')
    print(s_i)
    d = [
        [1, 2, 3],
        [1, 0, -1],
        [0, 1, 1]
    ]
    print(tool.select_row(d, 1))  # [2, 0, 1]
    tool.generator = False
    print(tool.select_ls(['遵义县', '虾子']))

    tool.string = '9994599945545599945'
    ts = tool.search('(45+)+')
    print(ts.start(), ts.end(), ts.value())
    print(tool.string)

    tool.string = 'are you fuck!'
    print(tool.replace('[0-9a-zA-Z]', ''))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/12/2 10:17
# @Author: Jtyoui@qq.com

"""
正则解析器
"""
try:
    import xml.etree.cElementTree as et
except ModuleNotFoundError:
    import xml.etree.ElementTree as et
import re


class RegexEngine:
    def __init__(self, xml, str_):
        """加载正则表。正则表为xml
        :param xml: 正则表的位置
        :param str_: 要匹配的字符串
        """
        self._string = str_
        self._root = et.parse(xml).getroot()
        self.re = ''
        self.data = []

    def select(self, tag):
        """根据xml的tag来实现不同的正则提取
        :param tag: xml的tag标签
        :return: 正则提取的数据
        """
        root = self._root.find(tag)
        attrib = root.attrib
        if attrib.get('part', 'False').lower() == 'true':
            self._part_tag(root)
            return list(filter(lambda x: x[1], self.data))
        else:
            sf = self._no_part(root)
            self.re = ''.join(self.data) + sf
            return re.findall(self.re, self._string)

    def _no_part(self, tags):
        """tag标签不分开抽取"""
        for tag in tags:
            if tag:
                if tag.attrib.get('must', 'true').lower() == 'true':
                    self.data.append(self.re)
                    self.re = ''
                    self.re = '(?:' + self._no_part(tag) + ')'
                else:
                    self.re = self._no_part(tag)
            else:
                attrib = tag.attrib
                text = tag.text.strip()
                if attrib.get('must', 'true').lower() == 'true':
                    self.re = '(?:' + text + ')'
                else:
                    self.re += '(?:' + text + ')?'
        return self.re

    def _part_tag(self, tags):
        """tag标签分开提取"""
        for tag in tags:
            if tag:
                self._part_tag(tag)
            else:
                self.data.append((tag.tag, re.findall(tag.text.strip(), self._string)))

    @property
    def string(self):
        return self._string

    @string.setter
    def string(self, str_):
        self._string = str_
        self.re, self.data = '', []



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/5 9:36
# @Author: Jtyoui@qq.com
from .SearchAlgorithm import *  # 搜索算法
from .MatchingAlgorithm import *  # 匹配算法
from .SortAlgorithm import *  # 排序算法
from .ReplaceAlgorithm import *  # 替换算法
from .TreeAlgorithm import *  # 创建树算法



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/29 15:08
# @Author: Jtyoui@qq.com
from enum import Enum


class Languages(Enum):
    """英语单词对应中文"""
    English = '英语'
    Japanese = '日语'
    Korean = '韩语'
    French = '法语'
    German = '德语'
    Spanish = '西班牙语'
    Chinese = '汉语'


if __name__ == '__main__':
    print(type(Languages.English))
    print(Languages.English)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/9 9:03
# @Author: Jtyoui@qq.com
from jtyoui.data.constant import MathSymbols

"""
联合概率:两件事情同时发生的概率
"""


def independent_joint(a, operation, b):
    """独立联合事件"""
    """
    :param operation: and==∩，or==|,比如:independent_joint(a,'and',b)等价于P(a∩b)
    """
    if operation == 'and' or operation == MathSymbols.intersection:
        return a * b
    elif operation == 'or' and operation == MathSymbols.condition:
        return a


def non_independent_joint(a, operation, b):
    """非独立联合事件"""
    if operation == 'and' or operation == MathSymbols.intersection:
        return a * non_independent_joint(b, MathSymbols.condition, a)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/1/24 0024
# @Email : jtyoui@qq.com
# @Software : PyCharm
import requests
from jtyoui.web import random
from urllib.parse import quote
import re
import struct

"""
    爬取搜狗只需要三步：
    第一步：搜索关键字。爬取关键字的下载链接
    第二步：筛选下载链接，进行下载
    第三步：将下载的搜狗文件转化为UTF-8格式
"""


class SoGou:
    def __init__(self, search):
        """
        初始化，也就是模糊搜索的第一步
        :param search:  关键字
        """
        self.header = {'User-Agent': random()}  # 设置UA
        # 将中文字转化为URL链接。注意搜狗将中文字进行的GBK编码。而不是UTF-8
        url_word = quote(search.encode('GBK'))
        url = 'https://pinyin.sogou.com/dict/search/search_list/%s/normal/' % url_word  # 搜索链接
        response = requests.get(url=url, headers=self.header)
        match = re.findall(url[24:] + '(.{1,3})">', response.text)  # 匹配下载页数
        max_page = max(map(lambda x: int(x), match)) if match else 1  # 选取最大的页数，如果没有页数返回1
        m = []  # 将匹配到下载链接
        for page in range(1, max_page + 1):
            response = requests.get(url=url + str(page), headers=self.header)
            match = re.findall(r'id=(.+)&name=(.+)"', response.text)  # 匹配下载链接
            m.extend(match)  # 将匹配到的下载链接装到链表中
        load_url = 'https://pinyin.sogou.com/d/dict/download_cell.php?id={0}&name={1}'  # 下载链接的格式
        # 将匹配到的，名字和ID映射到下载链接格式中
        self.load_url = map(lambda x: load_url.format(x[0], x[1]), m)

    def load_word(self, url_word=None):
        """
        下载搜狗文件
        :param url_word:  下载链接
        :return: 转化好的搜狗文件。返回格式的链表
        """
        load_ls = dict()  # 字典的键是下载词库的名字。字典的值是词库的内容
        if not url_word:
            for u in self.load_url:
                name = u[u.rfind('=') + 1:]  # 获取词库的名字
                response = requests.get(url=u, headers=self.header)
                ls = to_txt(response.content)  # 将其转化为UTF-8格式
                load_ls[name] = ls
        else:
            name = url_word[url_word.rfind('=') + 1:]
            response = requests.get(url=url_word, headers=self.header)
            load_ls[name] = to_txt(response.content)
        return load_ls  # 返回字典


def to_txt(data):
    ls_word = []  # 转化搜狗为UTF-8格式内容
    w = ''  # 每一个词条
    for i in range(0, len(data), 2):
        x = data[i:i + 2]  # 搜狗的UTF-8编码是两个字节
        t = struct.unpack('H', x)[0]  # 将其转化为无符号的短整形
        if 19968 < t < 40959 or t == 10:  # 判断是否是中文字符。10表示的是换行
            if t != 10:  # 不换行放在单个词条
                w += chr(t)
            elif t == 10 and len(w):  # 换行且不等于空
                ls_word.append(w)
                w = ''
    return ls_word[1:]  # 第一行是注释。不需要，去除第一行


if __name__ == '__main__':
    sg = SoGou('动物词汇大全【官方推荐】')  # 获取搜索关键字下的词库
    for s in sg.load_url:  # 遍历关键字下的URL
        txt = sg.load_word(s)  # 下载
        print(txt)  # 打印



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/22 11:44
# @Author: Jtyoui@qq.com

import zipfile
from jtyoui.data import letter_maps  # 声调映射表
from os import path


def load_pin_yin(tone=False):
    """加载拼音模型字典：参数：True 有声调， False没有声调
    :param tone:是否要加载声调
    :return: 拼音模型字典
    """
    d = {}
    file_zip = path.dirname(path.abspath(__file__))
    file_zip = path.join(path.dirname(file_zip), 'file_zip', 'py.zip')
    f = zipfile.ZipFile(file_zip)
    fp = f.read('py.txt')
    lines = fp.decode('utf-8').split('\n')
    for line in lines:
        v, k = line.split('#')
        if ',' in v:
            v = v[:v.index(',')]
        if tone:
            d.setdefault(k, v)
        else:
            vs = ''
            for v_ in v:
                vs += letter_maps[v_]
            d.setdefault(k, vs)
    return d


def chinese_to_pin_yin(pin_yin, string_):
    """
    将汉字转为拼音
    :param pin_yin:拼音模型
    :param string_: 汉字
    :return: 拼音列表
    """
    py = []
    for str_ in string_:
        data = pin_yin.get(str_)
        if data:
            py.append(data)
    return py


if __name__ == '__main__':
    load = load_pin_yin(True)
    print(chinese_to_pin_yin(load, '我喜欢你！'))
    load = load_pin_yin(False)
    print(chinese_to_pin_yin(load, '你好！世界'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/23 17:16
# @Author: Jtyoui@qq.com
import logging.config
import os
import configparser


# 参考文档：https://docs.python.org/zh-cn/3.7/library/logging.handlers.html?highlight=timedrotatingfilehandler


def get_log_config(config_path=None, custom_dir=None) -> configparser:
    """加载当前文件下的log.ini文件，默认日志文件夹在当前运训目录的logs下,
    如果要自定义文件夹，只需要将custom_dir定义该目录即可，修改目录下的日志文件夹只需要定义handlers即可，程序会自动寻找handlers下的args的值。
    [handlers]
    keys = consoleHandler,fileHandler,errorHandler
    :param config_path: 日志文件配置
    :param custom_dir:自定义日志文件夹
    :return: 日志文件配置对象
    """
    import sys  # 不能删除，程序会自动加载，勿删除
    cfg = configparser.RawConfigParser()
    if not config_path:
        config_path = os.path.dirname(__file__) + os.sep + 'log.ini'
    cfg.read(config_path)
    if custom_dir:
        if not os.path.exists(custom_dir):
            os.mkdir(custom_dir)
        handle = cfg.items('handlers')
        for _, v in handle:
            for vs in v.split(','):
                for key, value in cfg.items('handler_' + vs):
                    if key == 'args':
                        e = eval(value)
                        if isinstance(e[0], str):
                            es = custom_dir + os.sep + os.path.basename(e[0])
                            value = str((es, *e[1:]))
                            cfg.set('handler_' + vs, 'args', value)
    else:
        if not os.path.exists('./logs'):
            os.mkdir('./logs')
    return cfg


def set_log_file_config(config: configparser):
    """加载配置文件到内存"""
    logging.config.fileConfig(config)


def log_file_config():
    """默认加载配置文件"""
    c = get_log_config()
    set_log_file_config(c)


if __name__ == '__main__':
    log_file_config()  # 加载默认配置文件，如果要自定义，流程如下：c = get_log_config() 先对c对象进行修改，set_log_file_config(c)
    logging.info('默认加载到root下')

    info = logging.getLogger('info')
    info.info('日志文件写道info.log文件下')

    error = logging.getLogger('error')
    error.error('日志文件写道error.log文件下')



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/26 17:20
# @Author: Jtyoui@qq.com
"""排序算法"""


def bubbled_sort(ls):
    """冒泡算法"""
    length = len(ls)
    flag = True  # 判断是否要进行数据交换，True表示要进行
    k = length - 1  # 表示已经排序好的上界值,默认是列表的长度
    last = 0  # 记住上一次循环交换的位置。默认是开头
    for i in range(length):
        if not flag:
            break
        flag = False  # 每次循环都默认为不进行数据交换
        for j in range(k):
            if ls[j] > ls[j + 1]:
                ls[j], ls[j + 1] = ls[j + 1], ls[j]
                flag = True  # 要交换数据
                last = j  # 交换数据的位置
        k = last
    return ls


if __name__ == '__main__':
    import random
    import time

    s = []
    for _ in range(1_00):
        jr = random.randint(0, 1000)
        s.append(jr)
    start = time.time()
    bs = bubbled_sort(s)
    print(bs)
    print(time.time() - start)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/25 18:18
# @Author: Jtyoui@qq.com

from .AnalysisMath import AnalysisMath, cov  # 分析
from .matrix import Matrix  # 矩阵
from .target import *  # 分类预测



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/7 13:20
# @Author: Jtyoui@qq.com
import functools


@functools.singledispatch
def fun_generic(*args, **kwargs):
    """自定义泛型类型"""
    pass


@functools.singledispatch
def overwrite(func):
    """自定义泛型类型"""

    @functools.wraps
    def wraps(*args, **kwargs):
        func(*args, **kwargs)

    return wraps


if __name__ == '__main__':
    @fun_generic.register(str)  # 定义类型
    def _(*arg, **kwargs):
        print("str:", arg, kwargs)


    fun_generic('hello', 'world', key=1)


    @overwrite.register(int)
    def ws(age: int):
        print('int')


    @overwrite.register(str)
    def ws(age: str):
        print('str')


    overwrite('1')



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/5/7 19:48
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from urllib.parse import quote
from jtyoui.error import LibraryNotInstallError
from jtyoui.decorator import replace_regular
from jtyoui.web import header
import requests

# 记住下次换缓存
Response_Headers = """cookie: HJ_UID=a0752831-ba30-9486-ddc8-66bcbb7f303a; _REF=https://www.baidu.com/link?url%3DTvv2c125EbCEB2T5xBtSlQeMb4zSO1v2ZkeB8uvhFXacQdks-Z0OXCabLXXX-Wpa&wd%3D&eqid%3D9cbe02970005752c000000025cd1684b; _REG=www.baidu.com|; _SREG_3=www.baidu.com|; HJ_CST=0; HJ_CSST_3=0; _SREF_3=https://www.baidu.com/link?url%3DTvv2c125EbCEB2T5xBtSlQeMb4zSO1v2ZkeB8uvhFXacQdks-Z0OXCabLXXX-Wpa&wd%3D&eqid%3D9cbe02970005752c000000025cd1684b; TRACKSITEMAP=3%2C6%2C; HJ_SID=5b45f4b2-9ae1-4a93-803b-7e2d08faba78; HJ_SSID_3=e959e1f7-6b25-4c49-a26d-a914297c0f32
user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.131 Safari/537.36"""

word = {
    '自动词・五段/一类': '自Ⅰ',
    '形容动词/ナ形容词': '形Ⅱ',
    '他动词・一段/二类': '他Ⅱ',
    '他动词・五段/一类': '他Ⅰ',
    '自动词・一段/二类': '自Ⅱ',
    '形容词/イ形容词': '形Ⅰ',
    '自动词・サ变/三类': '自Ⅲ',
    '他动词・サ变/三类': '他Ⅲ',
    '他・サ变/三类': '他Ⅲ',
    '他・一段/二类': '他Ⅱ',
    '他・五段/一类': '他Ⅰ',
}


@replace_regular(r'（(.+)）|《(.+)》|[0-9].', '')
def sub_names(names):
    return names


def analysis(data_name, response_headers=Response_Headers):
    """分析单词、爬去小D网站：https://dict.hjenglish.com/jp/jc/%E3%81%AB%E3%81%8E%E3%82%8F%E3%81%86"""
    try:
        from bs4 import BeautifulSoup  # pip install bs4
    except Exception as _:
        raise LibraryNotInstallError('安装 pip install bs4')
    total = []
    ws = open(data_name + '.data', 'w', encoding='utf8')
    with open(data_name, encoding='utf8') as f:
        for line in f:
            lines = line[:-1]
            if len(lines) > 1 and (lines not in total):
                total.append(lines)
                name = lines
                url = F'https://dict.hjenglish.com/jp/jc/{quote(name)}'
                response = requests.get(url=url, headers=header(response_headers))
                data = response.text
                sp = BeautifulSoup(data, 'html.parser')
                pjm = sp.find(class_='pronounces')
                if pjm:
                    pjm = pjm.span.text[1:-1]
                else:
                    continue
                cx = sp.find(class_='simple')
                x = cx.h2
                if x:
                    x = x.text[1:-1]
                else:
                    continue
                m = cx.ul.text.replace('\n', '')
                m = sub_names(m).replace('。', '；')[:-1]
                if len(m) > 40:
                    continue
                if name != pjm:
                    name = "(" + name + ")"
                else:
                    name = None
                x = '<' + x + '>'
                for k, v in word.items():
                    x = x.replace(k, v)
                x = x.replace('词', '')
                if len(m) <= 1:
                    continue
                if name:
                    string = F'{pjm}{name}\t{x}\t{m}'
                else:
                    string = F'{pjm}\t{x}\t{m}'
                print(string)
                ws.write(string + '\n')
                ws.flush()


def cut(text_name):
    """从原始数据里面提取日语单词
    :param text_name:原始数据的文件地址
    :return:返回提取数据文本的地址
    """
    try:
        from janome.tokenizer import Tokenizer  # pip install janome
    except Exception as _:
        raise LibraryNotInstallError('安装 pip install janome')

    w = open(text_name + '.jp', 'w', encoding='utf8')
    t = Tokenizer()
    fp = open(text_name, encoding='utf-8')
    for token in t.tokenize(fp.read()):
        words = token.base_form
        words = words.replace(' ', '')
        if len(words) > 1:
            w.write(words + '\n')
    return text_name + '.jp'


if __name__ == '__main__':
    c = cut('2011年日语高考真题.txt')
    analysis(c)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/18 18:01
# @Author: Jtyoui@qq.com
from jtyoui.tools import pips
from jtyoui.data import word_nature


class TextSummary:
    def __init__(self, text, title):
        self.title = title
        self.text = text
        self.keywords = list()
        self.sentences = list()
        self.summary = list()
        self.cut = pips('jieba.posseg', 'jieba').cut  # 自动安装结巴包失败，请手动安装: pip install jieba
        self.extract_tags = pips('jieba.analyse', 'jieba').extract_tags  # 自动安装结巴包失败，请手动安装: pip install jieba

    def __setitem__(self, title, text):
        self.title = title
        self.text = text

    def _split_sentence(self):
        # 通过换行符对文档进行分段
        sections = self.text.split('\n')
        for section in sections:
            if section == '':
                sections.remove(section)

        # 通过分割符对每个段落进行分句
        for i in range(len(sections)):
            section = sections[i]
            text = ''
            k = 0
            for j in range(len(section)):
                char = section[j]
                text = text + char
                if char in '!。？' or j == len(section) - 1:
                    text = text.strip()
                    sentence = dict()
                    sentence['text'] = text
                    sentence['pos'] = dict()
                    sentence['pos']['x'] = i
                    sentence['pos']['y'] = k
                    # 将处理结果加入self.sentences
                    self.sentences.append(sentence)
                    text = ''
                    k = k + 1

        for sentence in self.sentences:
            sentence['text'] = sentence['text'].strip()
            if sentence['text'] == '':
                self.sentences.remove(sentence)

        # 对文章位置进行标注，通过mark列表，标注出是否是第一段、尾段、第一句、最后一句
        last_pos = dict()
        last_pos['x'] = 0
        last_pos['y'] = 0
        last_pos['mark'] = list()
        for sentence in self.sentences:
            pos = sentence['pos']
            pos['mark'] = list()
            if pos['x'] == 0:
                pos['mark'].append('first_section')
            if pos['y'] == 0:
                pos['mark'].append('first_sentence')
                last_pos['mark'].append('last_sentence')
            if pos['x'] == self.sentences[len(self.sentences) - 1]['pos']['x']:
                pos['mark'].append('last_section')
            last_pos = pos
        last_pos['mark'].append('last_sentence')

    def _calc_keywords(self):
        # 计算TF-IDF，取出排名靠前的20个词
        words_best = self.extract_tags(self.text, topK=20)
        # 提取第一段的关键词
        parts = self.text.lstrip().split('\n')
        first_part = ''
        if len(parts) >= 1:
            first_part = parts[0]
        words_best = words_best + self.extract_tags(first_part, topK=5)
        # 提取title中的关键词
        words_best = words_best + self.extract_tags(self.title, topK=3)
        # 将结果合并成一个句子，并进行分词
        text = ''
        for w in words_best:
            text += ' ' + w
        # 计算词性，提取名词和动词
        words = self.cut(text)
        keywords = list()
        for w in words:
            flag = w.flag
            word = w.word
            if flag.find('n') >= 0 or flag.find('v') >= 0:
                if len(word) > 1:
                    keywords.append(word)
        # 保留前20个关键词
        keywords = self.extract_tags(' '.join(keywords), topK=20)
        keywords = list(set(keywords))
        self.keywords = keywords

    def _calc_sentence_weight_by_keywords(self):
        # 计算句子的关键词权重
        for sentence in self.sentences:
            sentence['weightKeywords'] = 0
        for keyword in self.keywords:
            for sentence in self.sentences:
                if sentence['text'].find(keyword) >= 0:
                    sentence['weightKeywords'] += 1

    def _calc_sentence_weight_by_pos(self):
        # 计算句子的位置权重
        for sentence in self.sentences:
            mark = sentence['pos']['mark']
            weight_pos = 0
            if 'first_section' in mark:
                weight_pos = weight_pos + 2
            if 'first_sentence' in mark:
                weight_pos = weight_pos + 2
            if 'last_sentence' in mark:
                weight_pos = weight_pos + 1
            if 'last_section' in mark:
                weight_pos = weight_pos + 1
            sentence['weight_pos'] = weight_pos

    def _calc_sentence_weight_cue_words(self, index=None):
        # 计算句子的线索词权重
        if not index:
            index = word_nature()['adv_freq']
        for sentence in self.sentences:
            sentence['weightCueWords'] = 0
        for i in index:
            for sentence in self.sentences:
                if sentence['text'].find(i) >= 0:
                    sentence['weightCueWords'] = 1

    def _calc_sentence_weight(self, index=None):
        self._calc_sentence_weight_by_pos()
        self._calc_sentence_weight_cue_words(index)
        self._calc_sentence_weight_by_keywords()
        for sentence in self.sentences:
            sentence['weight'] = sentence['weight_pos'] + 2 * sentence['weightCueWords'] + sentence['weightKeywords']

    def calc_summary(self, ratio=0.1, index=None):
        # 清空变量
        self.keywords.clear()
        self.sentences.clear()
        self.summary.clear()

        # 调用方法，分别计算关键词、分句，计算权重
        self._calc_keywords()
        self._split_sentence()
        self._calc_sentence_weight(index)

        # 对句子的权重值进行排序
        self.sentences = sorted(self.sentences, key=lambda k: k['weight'], reverse=True)

        # 根据排序结果，取排名占前X%的句子作为摘要
        for i in range(len(self.sentences)):
            if i < ratio * len(self.sentences):
                sentence = self.sentences[i]
                self.summary.append(sentence['text'])

        return self.summary


if __name__ == '__main__':
    data = """6月17日22时55分，四川长宁县发生6.0级地震，震源深度16千米。地震发生两个小时后，离震中较近的四川省宜宾市珙县
    巡场镇宜宾市矿山急救医院迎来第一个新生儿。医生在余震和医院房屋出现损毁的情况下顶住压力和风险，为产妇接生，母子平安。"""
    ts = TextSummary(data, title='长宁县地震')
    print(ts.calc_summary())



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/1 15:53
# @Author: Jtyoui@qq.com
from collections import Counter
import json


# 没有在vocab是1
# 填充是0
# 结尾符是2
def read_test(path):
    vocab = json.load(fp=open('./vocab.json'))
    test = []
    with open(path) as fp:
        for line in fp:
            lines = line.strip().split('_')
            m = map(lambda x: vocab.get(x, 1), lines)
            test.append(list(m))
    return test


def read_train(tag, path):
    train = []
    test = []
    with open(path) as fp:
        data = fp.read().split('\n\n')[:-1]
        vocab = json.load(fp=open('./vocab.json'))
        for ds in data:
            d, t = [], []
            lines = ds.split('\n')
            for line in lines:
                k, v = line.split('\t')
                d.append(vocab.get(k, 1))
                t.append([tag[v]])
            train.append(d)
            test.append(t)
    return train, test


def get_vocab(path):
    v = []
    with open(path) as fp:
        for line in fp:
            data = line.strip().split('_')
            v.extend(data)
    c = Counter(v)
    f = filter(lambda x: x[1] >= 10, c.items())
    s = sorted(f, key=lambda x: x[1], reverse=True)
    vocab = {}
    for index, (k, _) in enumerate(s, start=3):
        vocab[k] = index
    json.dump(vocab, fp=open('./vocab.json', 'w'))
    return vocab


if __name__ == '__main__':
    get_vocab(path=None)
    tags = {'O': 0,
            'B-a': 1, 'I-a': 2,
            'B-b': 3, 'I-b': 4,
            'B-c': 5, 'I-c': 6
            }
    trains, tests = read_train(tags, path=None)
    print(trains)
    print(tests)
    print(read_test(path=None))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/18 9:32
# @Author: Jtyoui@qq.com

from .ChebyshevDistance import chebyshev_distance  # 切比雪夫距离
from .CosineDistance import cosine_distance, cosine  # 余弦距离
from .EditDistance import edit_distance  # 编辑距离
from .EuclideanDistance import euclidean_distance  # 欧氏距离
from .HammingDistance import ham_distance, simHash_similarity  # 海明距离
from .MahalanobisDistance import mahalanobis_distance  # 马氏距离
from .ManhattanDistance import manhattan_distance  # 曼哈顿距离
from .MinkowskiDistance import minkowski_distance  # 闵可夫斯基距离
from .JaccardDistance import jaccard_set_distance, jaccard_distance  # 杰卡德距离
from .BrayCurtisDistance import bray_curtis_distance  # 布雷柯蒂斯距离



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/18 9:34
# @Author: Jtyoui@qq.com
from jtyoui.statistics.analysis import Matrix, cov, AnalysisMath

"""
马氏距离
马氏距离是由印度统计学家马哈拉诺比斯提出的，
表示数据的协方差距离。它是一种有效的计算两个未知样本集的相似度的方法。与欧氏距离不同的是，
它考虑到各种特性之间的联系（例如：一条关于身高的信息会带来一条关于体重的信息，因为两者是有关联的），
并且是尺度无关的(scale-invariant)，即独立于测量尺度。对于一个均值为μ，协方差矩阵为Σ的多变量向量，
其马氏距离为sqrt( (x-μ)'Σ^(-1)(x-μ) )。
"""


def mahalanobis_distance(matrix, coordinate):
    """
    求解马氏距离
    :param matrix: 矩阵列表
    :param coordinate: 坐标
    :return result: 两个点的马氏距离
    """
    # 马氏距离要求样本数要大于维数，否则无法求协方差矩阵
    # 此处进行转置，表示10个样本，每个样本2维
    s = Matrix(matrix).t
    ana = AnalysisMath()
    mean, c, distance = [], [], []
    for index, i in enumerate(s):
        c.append([cov(i, j) for j in s])
    s_i = Matrix(c).i  # 协方差矩阵的逆矩阵
    for data in s:
        mean.append(ana.expect(data))
    distance.append([i - j for i, j in zip(coordinate, mean)])
    ma = Matrix(Matrix(distance) * s_i) * Matrix(distance).t
    return pow(ma[0][0], 0.5)


if __name__ == '__main__':
    mat = [
        [3, 4],
        [5, 6],
        [2, 8],
        [8, 4]
    ]
    print(mahalanobis_distance(mat, [3, 4]))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/5/12 16:16
# @Email  : jtyoui@qq.com
# @Software: PyCharm

from jtyoui.file_zip import load_zip
from jtyoui.web import random
import requests
import re
from html.parser import HTMLParser


class _GetData(HTMLParser):

    def __init__(self):
        HTMLParser.__init__(self)
        self.weather_7d = False
        self.name = False
        self.addr = ''
        self.data_7d = []
        self.string = ''

    def handle_starttag(self, tag, attrs):
        if len(attrs) > 0:
            attr = attrs[0]
        else:
            return
        if tag == 'ul' and 't clearfix' in attr:
            self.weather_7d = True
        elif tag == 'div' and 'crumbs fl' in attr:
            self.name = True

    def handle_endtag(self, tag):
        if tag == 'ul':
            self.weather_7d = False
            if self.string:
                self.data_7d.append(self.string[:-1] + '风力')
                self.string = ''
        elif tag == 'div':
            self.name = False

    def handle_data(self, data):
        if self.weather_7d:
            data = data.replace('\n', '').replace(u'\xa0', '')
            if data:
                if '（' in data:
                    if self.string:
                        self.data_7d.append(self.string[:-1] + '风力')
                    self.string = data
                else:
                    self.string += data + ' '
        elif self.name:
            data = re.sub(r'[\r\n \t\xa0>]', '', data)
            self.addr += data


class WeatherForecast:
    def __init__(self):
        """天气预报，使用方法
        >>>import jtyoui
        >>>import pprint
        >>>w = jtyoui.WeatherForecast()
        >>>w.set_city('九龙')
        >>>pprint.pprint(w.get_today_weather())  # 获得当天 天气预报
        >>>pprint.pprint(w.get_7day_weather())  # 获得7天天气预报
        >>>pprint.pprint(w.get_15day_weather())  # 获得15天天气预报
        """
        self.weather = load_zip('city.zip', 'city.txt')
        self.codes = []

    def set_city(self, city):
        """
        输入一个城市名字
        :param city: 城市名字
        """
        if self.codes:
            self.codes.clear()
        for names in self.weather:
            code, city_ = names.split(',')
            if city_ in city:
                self.codes.append(code)

    def get_today_weather(self):
        """获得当天 天气预报"""
        total = []
        for code in self.codes:
            url = F'http://www.weather.com.cn/weather1d/{code}.shtml'
            response = requests.get(url, headers={'User-Agent': random()})
            response.encoding = 'utf-8'
            data = response.text
            d = re.findall(r'<input type="hidden" id="hidden_title" value="(.+)" />', data)
            gd = _GetData()
            gd.feed(data)
            total.append({gd.addr: d})
        return total

    def get_7day_weather(self):
        """获得7天 天气预报"""
        total = []
        for code in self.codes:
            url = F'http://www.weather.com.cn/weather/{code}.shtml'
            response = requests.get(url, headers={'User-Agent': random()})
            response.encoding = 'utf-8'
            gd = _GetData()
            gd.feed(response.text)
            total.append({gd.addr: gd.data_7d})
        return total

    def get_15day_weather(self):
        """获得15天 天气预报"""
        t_7d = self.get_7day_weather()
        for code in self.codes:
            url = F'http://www.weather.com.cn/weather15d/{code}.shtml'
            response = requests.get(url, headers={'User-Agent': random()})
            response.encoding = 'utf-8'
            gd = _GetData()
            gd.feed(response.text)
            for t in t_7d:
                if gd.addr in t:
                    t[gd.addr].extend(gd.data_7d)
        return t_7d


if __name__ == '__main__':
    import pprint

    w = WeatherForecast()
    w.set_city('九龙')
    pprint.pprint(w.get_today_weather())  # 获得当天 天气预报
    pprint.pprint(w.get_7day_weather())  # 获得7天天气预报
    pprint.pprint(w.get_15day_weather())  # 获得15天天气预报



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/9/25 10:12
# @Author: Jtyoui@qq.com
import jtyoui
import click
import os
import pprint

DESC = jtyoui.__description__ + '更新时间为20' + jtyoui.__version__.replace('.', '-') + '。'
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help', '--h'])


def split_doc(func, name):
    f = getattr(jtyoui, func)
    doc = f.__doc__
    docs = doc.strip().split('\n')
    for x in docs:
        if ':param ' + name in x:
            return x
    return '联系作者写注释！'


@click.group(context_settings=CONTEXT_SETTINGS, help=DESC)
@click.version_option(version='1.0.0', help='当前版本')
def main(): ...


@main.add_command
@click.command(help='成词发现')
@click.option('-f', type=str, help='文本地址')
@click.option('-n', type=int, help='成词最大粒度')
@click.option('-t', type=int, help='key出现的次数')
@click.option('-q', type=float, help='过滤的频率')
@click.option('-c', type=float, help='过滤凝聚度')
@click.option('-r', type=float, help='过滤自由度')
def neologism(f, n, t, q, c, r):
    print('等待中，正在训练数据.......')
    from jtyoui.word.neologism import mains as m
    m(f, n, t, q, c, r)
    print('训练完毕！')


@main.add_command
@click.command(help='创建Flask Docker项目文件')
@click.option('-f', type=str, help='创建Flask Docker项目的路径')
def docker_file(f):
    print('等待中.......')
    fp = os.path.abspath(f)
    jtyoui.create_docker_project(fp)
    print('创建成功！')


@main.add_command
@click.command(help='查询天气预报')
@click.option('-name', type=str, help='城市名字')
@click.option('-day', type=int, default=1, help='查询的天数：【1,7,15】、默认是当天。')
def weather(name, day):
    w = jtyoui.WeatherForecast()
    w.set_city(name)
    if day == 15:
        days = w.get_15day_weather()  # 获得当天 天气预报
    elif day == 7:
        days = w.get_7day_weather()  # 获得7天天气预报
    else:
        days = w.get_today_weather()  # 获得15天天气预报
    pprint.pprint(days)


@main.add_command
@click.command(help='更新pip包')
def upload():
    os.system('python setup.py sdist bdist_wheel')
    os.system('python -m twine upload dist/*')
    print('更新完毕！')


if __name__ == "__main__":
    main()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/7 16:01
# @Author: Jtyoui@qq.com
import numpy as np


def sigmoid(x):
    """Sigmoid函数"""
    return 1.0 / (1 + np.exp(-x))


def error_rate(h, label):
    """计算当前的损失函数值
    :param h:预测值
    :param label:实际值
    :return:错误率
    """
    m = np.shape(h)[0]
    sum_err = 0
    for i in range(m):
        if 0 < h[i, 0] < 1:
            sum_err -= label[i, 0] * np.log(h[i, 0]) + (1 - label[i, 0]) * np.log(1 - h[i, 0])
        else:
            sum_err -= 0
    return sum_err / m


def get_cost(predict: np.mat, label: np.mat) -> float:
    """计算损失函数的值
    :param predict:预测值
    :param label:标签
    :return:损失函数的值
    """
    m = len(predict)
    error = 0.0
    for i in range(m):
        error -= np.log(sigmoid(predict[i, 0] * label[i, 0]))
    return error


def line_regression_a_b(x: np.array, y: np.array):
    """求解线性回归的a和β值。即y=b+a*x
    :param x: 数据特征值：应该是两维度。即：[[],[],[]]
    :param y: 数据预测值;一维度。即：[]
    :return: a值、和b值
    """
    var = np.var(x, ddof=1)  # 贝塞尔校正,方差
    cov = np.cov(x.transpose(), y)[0][1]  # 协方差
    x_ = np.mean(x)
    y_ = np.mean(y)
    a = cov / var
    b = y_ - a * x_
    return a, b


if __name__ == '__main__':
    a_, b_ = line_regression_a_b(np.array([[6], [8], [10], [14], [18]]), np.array([7, 9, 13, 17.5, 18]))
    print(np.array([[12]]) * a_ + b_)  # [[13.68103448]]



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/13 17:48
# @Author: Jtyoui@qq.com
import matplotlib.pyplot as plt
import matplotlib as mpl
from sklearn.datasets import load_breast_cancer
from sklearn.linear_model import LogisticRegression
from sklearn.neighbors import KNeighborsClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.model_selection import train_test_split

"""
三种分类算法示意图展示
"""

mpl.rcParams['font.sans-serif'] = ['FangSong']  # 指定默认字体
mpl.rcParams['axes.unicode_minus'] = False  # 解决保存图像是负号'-'显示为方块的问题

x, y = load_breast_cancer(return_X_y=True)
length = len(x) - 100
print(x)
print(y)
lr_scores, nb_scores, knn_scores = [], [], []
sizes = range(100, length, 30)
for size in sizes:
    x_train, x_test, y_train, y_test = train_test_split(x, y, train_size=size, random_state=31)
    print(len(x_train), size)
    lr = LogisticRegression()
    lr.fit(x_train, y_train)
    lr_score = lr.score(x_test, y_test)
    lr_scores.append(lr_score)

    nb = GaussianNB()
    nb.fit(x_train, y_train)
    nb_score = nb.score(x_test, y_test)
    nb_scores.append(nb_score)

    knn = KNeighborsClassifier()
    knn.fit(x_train, y_train)
    knn_score = knn.score(x_test, y_test)
    knn_scores.append(knn_score)

plt.plot(sizes, lr_scores, label='逻辑回归', linestyle='--')

plt.plot(sizes, nb_scores, label='朴素贝叶斯', linestyle=':')

plt.plot(sizes, knn_scores, label='KNN近邻')

plt.xlabel('训练的数据量')
plt.ylabel('预测的准确度')
plt.legend()
plt.show()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/6/1 22:34
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from itertools import combinations_with_replacement, permutations, combinations
from collections.abc import Iterable
import random
import string
import collections
import re
import os
import unicodedata
import jtyoui

_special = "#$%&@"


# 随机选择字母:
def random_char(number=1):
    """随机选择字母:number是生成个数"""
    ls = random.choices(string.ascii_letters, k=number)
    return ''.join(ls)


# 随机选择小写字母:
def random_lower_char(number=1):
    """随机选择小写字母:number是生成个数"""
    ls = random.choices(string.ascii_lowercase, k=number)
    return ''.join(ls)


# 随机选择大写字母:
def random_upper_char(number=1):
    """随机选择大写字母:number是生成个数"""
    ls = random.choices(string.ascii_uppercase, k=number)
    return ''.join(ls)


# 随机选择数字:
def random_digits(number=1):
    """随机选择数字:number是生成个数"""
    ls = random.choices(string.digits, k=number)
    return ''.join(ls)


# 随机选择特殊字符:
def random_special(number=1):
    """随机选择特殊字符:number是生成个数"""
    ls = random.choices(_special, k=number)
    return ''.join(ls)


def flag_contain_subset(str_: str, ls: list) -> bool:
    """输入一个字符串判断字符串的子集是否在ls列表中"""
    v = (True if subset in str_ else False for subset in ls)
    return any(v)


def contain_subset(str_: str, ls: list) -> (bool, list):
    """
    输入一个字符串判断字符串的子集是否在ls列表中,并且返回子集列表
    :param str_: 字符串
    :param ls: 字符串列表
    :return: 存在返回True。不存在返回False。都会返回list列表
    """
    v = [subset for subset in ls if subset in str_]
    return any(v), v


def max_str(ls: list):
    """统计字符串列表出现字符串最多的字符串"""
    c = collections.Counter(ls)
    return max(c.keys(), key=c.get)


def contain_list_subset(str_: str, ls: list) -> (bool, list):
    """
    输入一个字符串判断字符串是否属于某个列表的子集。例如：str_：贵州，ls：[贵州省，遵义市]，那么贵州属于ls某个字符串的子集
     :param str_: 字符串
    :param ls: 字符串列表
    :return: 存在返回True。不存在返回False。都会返回list列表
    """
    v = [subset for subset in ls if str_ in subset]
    return any(v), v


def char_number_split(str_: str, number: int):
    """根据字符串个数来分割字符串"""
    while str_:
        yield str_[:number]
        str_ = str_[number:]


def split(re_, str_, flag=0, max_split=0) -> list:
    """支持正则分割
    :param re_:正则表达式
    :param str_:字符串
    :param flag: re.search(re_, self.string, flag), 默认flag=0
    :param max_split: 最大分割数量
    """
    return re.split(pattern=re_, string=str_, maxsplit=max_split, flags=flag)


def replace(re_, repl, string_, count=0, flags=0):
    """支持正则替换"""
    return re.sub(re_, repl, string_, count, flags)


def remove_subset(ls: list) -> list:
    """去除列表中的子集。比如：['aa','a','ab'] --> ['aa','ab']"""
    ls = sorted(ls, key=lambda x: len(x), reverse=True)
    total = []
    for subset in ls:
        if subset not in total:
            flag = True
            for word in total:
                if subset in word:
                    flag = False
                    break
            if flag:
                total.append(subset)
    return total


def rm_empty_dir(dir_path):
    """删除空目录"""
    for root, dirs, files in os.walk(dir_path):
        if not os.listdir(root):
            os.rmdir(root)


def combination(ls: iter, number=2) -> list:
    """组合：不重复"""
    c = combinations(ls, number)
    return list(c)


def combination_repeat(ls: iter, number=2) -> list:
    """组合：可重复"""
    c = combinations_with_replacement(ls, number)
    return list(c)


def permutation(ls: iter, number=2) -> list:
    """排列"""
    c = permutations(ls, number)
    return list(c)


def is_chinese(char: str) -> bool:
    """判断一个字符是否是中文
    :param char: 一个字符
    :return: 是中文返回真，否则是假
    """
    if len(char) == 1 and '\u4e00' < char < '\u9fa5':
        return True
    return False


def print_heart(s='♥'):
    """输出一个心脏的符号
    :param s: 字符串
    :return: 心脏的格式
    """
    ls = []
    s += ' '
    for y in range(15, -15, -1):
        flag = []
        for x in range(-30, 30):
            x1 = x * 0.04
            y1 = y * 0.1
            m = (x1 ** 2 + y1 ** 2 - 1) ** 3 - x1 ** 2 * y1 ** 3  # 心脏公式
            flag.append(s[(x - y) % len(s)] if m <= 0 else ' ')
        ls.append(''.join(flag) + '\n')
    print("\033[5;31m" + ''.join(ls) + '\033[0m')
    return ls


def get_argCount(func) -> int:
    """获取函数对象的参数个数

    def sum(a,b):
        return(a+b)
    print(sum.__code__.co_argcount)
    # 2
    #输出的函数参数个数
    print(sum.__code__.co_varnames)
    # ('a', 'b')
    #这里会输出函数用到的所有变量名，不只是参数名

    print(sum.__defaults__)
    # None
    # 返回参数的初始值

    import inspect
    inspect.getargspec(sum)
    # ArgSpec(args=['a', 'b'], varargs=None, keywords=None, defaults=None)
    :param func: 函数对象
    :return: 函数对象的参数个数
    """
    return func.__code__.co_argcount


def strip(data: str, re_) -> str:
    """支持正则去除data中的数据。类似于str.strip()函数
    :param data: 数据
    :param re_:去除data中的数据
    :return: 去除后的数据
    """

    @jtyoui.replace_regular(re_, '')
    def clean(data_):
        return data_

    return clean(data)


def find_unicodedata_name(data: str) -> list:
    """查询Unicode编码中的名字
    ♠ == BLACK SPADE SUIT
    \N{BLACK SPADE SUIT} == ♠
    :param data: 字符串
    :return: 字符的Unicode名字列表
    """
    ls = []
    for i in data:
        ls.append(unicodedata.name(i))
    return ls


def join(chars: str, obj: Iterable) -> str:
    """同str.join函数一样，只不过数字会自动转为字符串
    :param chars: 要拼接的字符串
    :param obj: 拼接对象
    :return: 字符串
    """
    o = (str(i) for i in obj)
    return chars.join(o)


def find(str_: str, re_: str) -> list:
    """功能类似于str.find(),但是支持正则表达式"""
    f, flag = [], 0
    x = re.findall(re_, str_)
    for v in x:
        index = str_.find(v, flag)
        flag = index + 1
        f.append(index)
    return f


def key_value_re(key: list, value: list, value_re: str = None, key_re: str = None) -> list:
    """根据value值的索引获取key或者根据key的索引获取到value。
    :param key:k值。['a','b']
    :param value:v值。[0,1]
    :param value_re: 根据值的正则获取key。比如：01正则表达式获取到ab
    :param key_re:同理。根据key的正则。获取到值。比如：ab正则表达式。返回01
    """
    if key_re is None and value_re is None:
        raise TypeError('value_re和key_re必须写一个')
    if len(key) == len(value):
        if value_re:
            tool = jtyoui.Tool(join('', key))
            return tool.index_select_string(join('', value), value_re)
        else:
            tool = jtyoui.Tool(join('', value))
            return tool.index_select_string(join('', key), key_re)
    else:
        "这里当key和value不相等时。暂时没有想到怎么处理，比如：['我', '叫', '刘', '万', '光'], [6, 6, 10, 11, 11],这种情况"
    return []


if __name__ == '__main__':
    print(random_char(4))
    print(random_lower_char(4))
    print(random_special(4))
    print(random_upper_char(4))
    print(random_digits(4))
    print(flag_contain_subset('我家住在北京', '家住、诉求、请求'.split('、')))
    print(contain_subset('我家住在北京', '家住、诉求、请求'.split('、')))
    print(max_str(['a', 'a', 'a', 'b', 'c', 'd', 'd']))
    print(contain_list_subset('贵州', ['贵州省', '遵义市', '贵州省贵阳市']))
    for cns in char_number_split('我家住在北京', 4):
        print(cns)
    print(split('[.,，。]', '我家组在北京。我去玩，啊'))
    print(remove_subset(['aa', 'a', 'ab'] * 1_0000))
    print(combination(range(1, 20)))
    print(combination_repeat(range(1, 20)))
    print(permutation(range(1, 20)))
    print(is_chinese('张'), is_chinese('a'))
    print_heart()
    print(get_argCount(contain_subset))
    print(strip('张a伟', 'a'))
    print(find_unicodedata_name('♠'))
    print(key_value_re(['我', '叫', '刘', '万', '光'], [6, 6, 0, 1, 1], value_re='01+'))
    print(find('abc', 'a|b'))  # 查找a或者b的索引



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/5/9 11:48
# @Author: Jtyoui@qq.com

"""
SC (solar calendar)：阳历
阳历转农历
"""

import time
from jtyoui.data import holiday, week_to_chinese, chinese_mon


class Lunar:
    """lun = Lunar(year=2018, month=1, day=2)   print(lun)  # 二零一九年 七月 十四 星期四 无
    print(lun.y)  # 农历的年,中文字符 二零一九
    print(lun.year)  # 农历的年，阿拉伯数字 2019
    print(lun.m)  # 农历的月份 中文字符 七
    print(lun.month)  # 农历的月份 阿拉伯字符 7
    print(lun.d)  # 农历的日期 中文字符 十四
    print(lun.day)  # 阳历的日期 阿拉伯数字 15 ，注意。和农历不一样
    print(lun.w)  # 星期几 中文字符
    print(lun.week)  # 星期几、英文字符
    print(lun.h)  # 节日
    print(lun)  # 二零一九年 七月 十四 星期四 无
    """

    def __init__(self, year, month, day):
        """
            1980年的数据是： 0x095b0
            二进制：0000 1001 0101 1011 0000
            1-4: 表示当年有无闰年，有的话，为闰月的月份，没有的话，为0。
            5-16：为除了闰月外的正常月份是大月还是小月，1为30天，0为29天。
            注意：从1月到12月对应的是第16位到第5位。
            17-20：表示闰月是大月还是小月，仅当存在闰月的情况下有意义。
        """
        self.date = '{0}-{1:0>2}-{2:0>2}'.format(year, month, day)
        if year > 2050 or year < 1990:
            raise ValueError('输入的范围在1990-2050年之间')
        self.year = 0
        self.month = 0
        self.day = 0
        self.week = 0
        self.leap = False
        self.chinese_week = week_to_chinese
        self.chinese_number = chinese_mon
        self.lunar_info = [
            0x04bd8, 0x04ae0, 0x0a570, 0x054d5, 0x0d260, 0x0d950, 0x16554, 0x056a0, 0x09ad0, 0x055d2,
            0x04ae0, 0x0a5b6, 0x0a4d0, 0x0d250, 0x1d255, 0x0b540, 0x0d6a0, 0x0ada2, 0x095b0, 0x14977,
            0x04970, 0x0a4b0, 0x0b4b5, 0x06a50, 0x06d40, 0x1ab54, 0x02b60, 0x09570, 0x052f2, 0x04970,
            0x06566, 0x0d4a0, 0x0ea50, 0x06e95, 0x05ad0, 0x02b60, 0x186e3, 0x092e0, 0x1c8d7, 0x0c950,
            0x0d4a0, 0x1d8a6, 0x0b550, 0x056a0, 0x1a5b4, 0x025d0, 0x092d0, 0x0d2b2, 0x0a950, 0x0b557,
            0x06ca0, 0x0b550, 0x15355, 0x04da0, 0x0a5d0, 0x14573, 0x052d0, 0x0a9a8, 0x0e950, 0x06aa0,
            0x0aea6, 0x0ab50, 0x04b60, 0x0aae4, 0x0a570, 0x05260, 0x0f263, 0x0d950, 0x05b57, 0x056a0,
            0x096d0, 0x04dd5, 0x04ad0, 0x0a4d0, 0x0d4d4, 0x0d250, 0x0d558, 0x0b540, 0x0b5a0, 0x195a6,
            0x095b0, 0x049b0, 0x0a974, 0x0a4b0, 0x0b27a, 0x06a50, 0x06d40, 0x0af46, 0x0ab60, 0x09570,
            0x04af5, 0x04970, 0x064b0, 0x074a3, 0x0ea50, 0x06b58, 0x055c0, 0x0ab60, 0x096d5, 0x092e0,
            0x0c960, 0x0d954, 0x0d4a0, 0x0da50, 0x07552, 0x056a0, 0x0abb7, 0x025d0, 0x092d0, 0x0cab5,
            0x0a950, 0x0b4a0, 0x0baa4, 0x0ad50, 0x055d9, 0x04ba0, 0x0a5b0, 0x15176, 0x052b0, 0x0a930,
            0x07954, 0x06aa0, 0x0ad50, 0x05b52, 0x04b60, 0x0a6e6, 0x0a4e0, 0x0d260, 0x0ea65, 0x0d530,
            0x05aa0, 0x076a3, 0x096d0, 0x04bd7, 0x04ad0, 0x0a4d0, 0x1d0b6, 0x0d250, 0x0d520, 0x0dd45,
            0x0b5a0, 0x056d0, 0x055b2, 0x049b0, 0x0a577, 0x0a4b0, 0x0aa50, 0x1b255, 0x06d20, 0x0ada0
        ]
        self.lunar()

    def lunar(self):
        date_time = time.strptime(self.date, '%Y-%m-%d')
        # 25538是1900-01-01到1970-01-01的天数，86400是一年的秒数
        total_day = int(time.mktime(date_time) / 86400) + 25538

        # 计算当天是农历第几天
        lunar_year, year_day = 0, 0
        while lunar_year < 2050 - 1900 and total_day > 0:
            year_day = self.year_days(lunar_year)
            total_day -= year_day
            lunar_year += 1

        if total_day < 0:
            total_day += year_day
            lunar_year -= 1

        # 农历年份
        self.year = lunar_year + 1900
        leap_month = self.leap_months(lunar_year)  # 闰哪个月, 1 - 12
        # 用当年的天数offset, 逐个减去每月（农历）的天数，求出当天是本月的第几天
        lunar_month, month_day = 1, 0
        while lunar_month < 13 and total_day > 0:
            # 闰月
            if leap_month > 0 and lunar_month == (leap_month + 1) and (not self.leap):
                lunar_month -= 1
                self.leap = True
                month_day = self.leap_days(lunar_year)
            else:
                month_day = self.month_days(lunar_year, lunar_month)
            total_day -= month_day
            # 解除闰月
            if self.leap and lunar_month == (leap_month + 1):
                self.leap = False
            lunar_month += 1

        # offset为0时，并且刚才计算的月份是闰月，要校正
        if total_day == 0 and leap_month > 0 and lunar_month == leap_month + 1:
            leap = False if self.leap else True
            if leap:
                lunar_month -= 1
        # offset小于0时，也要校正
        if total_day < 0:
            total_day += month_day
            lunar_month -= 1
        self.month = lunar_month
        self.day = total_day + 1
        self.week = time.strftime('%a', date_time)

    def year_days(self, lunar_year):
        i, sum_day = 0x8000, 348
        while i > 0x8:
            if (self.lunar_info[lunar_year] & i) != 0:
                sum_day += 1
            i >>= 1
        return sum_day + self.leap_days(lunar_year)

    def leap_months(self, lunar_year):
        return int(self.lunar_info[lunar_year] & 0xf)

    def leap_days(self, lunar_year):
        if self.leap_months(lunar_year) != 0:
            return 30 if (self.lunar_info[lunar_year] & 0x10000) != 0 else 29
        return 0

    def month_days(self, lunar_year, lunar_month):
        return 29 if (self.lunar_info[lunar_year] & (0x10000 >> lunar_month)) == 0 else 30

    def get_china_day(self, d):
        chinese_ten = ['初', '十', '廿', '卅']
        n = 9 if d % 10 == 0 else (d % 10 - 1)
        if d == 10:
            return '初十'
        elif n == 0:
            return chinese_ten[d // 10] + '一'
        else:
            return chinese_ten[d // 10] + self.chinese_number[n]

    def get_holiday(self, date):
        x = holiday.get(date[5:])
        if x:
            return x
        else:
            z = holiday.get(self.chinese_number[self.month - 1] + '月' + self.get_china_day(self.day))
            return z if z else '无'

    def __str__(self):
        """
        :return: 年 月 日 星期 节日(没有是无)
        """
        if self.leap:
            return self.y + '年 闰' + self.m + '月 ' + self.d + ' ' + self.w + ' 节日：' + self.h
        else:
            return self.y + '年 ' + self.m + '月 ' + self.d + ' ' + self.w + ' 节日：' + self.h

    def __getattr__(self, item):
        if item == 'y':
            y = ''
            for i in range(3, -1, -1):
                y += self.chinese_number[self.year // (10 ** i) % 10]
            return y.replace('正', '一')
        elif item == 'm':
            return self.chinese_number[self.month]
        elif item == 'd':
            return self.get_china_day(self.day)
        elif item == 'w':
            return self.chinese_week.get(self.week)
        elif item == 'h':
            return self.get_holiday(self.date)

    def to_sc(self):
        """年 月 日 """
        if self.leap:
            return str(self.year) + '年闰' + str(self.month) + '月' + str(self.day) + '日'
        else:
            return str(self.year) + '年' + str(self.month) + '月' + str(self.day) + '日'

    def __repr__(self):
        return self.__str__()


SC = Lunar

if __name__ == '__main__':
    lun = Lunar(year=2017, month=7, day=27)
    print(lun.y)  # 农历的年,中文字符 二零一九
    print(lun.year)  # 农历的年，阿拉伯数字 2019
    print(lun.m)  # 农历的月份 中文字符 七
    print(lun.month)  # 农历的月份 阿拉伯字符 7
    print(lun.d)  # 农历的日期 中文字符 十四
    print(lun.day)  # 阳历的日期 阿拉伯数字 15 ，注意。和农历不一样
    print(lun.w)  # 星期几 中文字符
    print(lun.week)  # 星期几、英文字符
    print(lun.h)  # 节日
    print(lun)  # 二零一九年 七月 十四 星期四 无
"""
二零一九
2019
七
7
十四
15
四
Thu
无
2019年 七月 十四 星期四 无
"""



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/9 0009
# @Email : jtyoui@qq.com
# @Software : PyCharm
from jtyoui.error import LibraryNotInstallError
from jtyoui.tools import pips
from jtyoui.file_zip import file_zip_path
from tkinter import filedialog, messagebox
import os
import tkinter
import glob

try:
    import fitz  # 安装 pip install PyMuPDF
except ModuleNotFoundError:
    try:
        fitz = pips('fitz', 'PyMuPDF')  # 自动安装
    except ModuleNotFoundError:
        raise LibraryNotInstallError("安装 pip install PyMuPDF")

DIRS, FILE = '', ''  # 文件夹地址,pdf文件地址


def select_pdf():
    global FILE
    FILE = filedialog.askopenfilename(filetypes=(("PDF file", "*.pdf"),))  # 获得pdf文件地址


def select_dir():
    global DIRS
    DIRS = filedialog.askdirectory()  # 获得文件夹地址


def start():
    global DIRS, FILE
    if DIRS:
        image_pdf(DIRS)  # 执行图片转pdf
        DIRS = ''
    elif FILE:
        if not os.path.exists(FILE[:-4]):
            os.mkdir(FILE[:-4])
        pdf_image(FILE)  # 执行pdf转照片
        FILE = ''
    else:
        messagebox.showwarning('警告', '先选择在执行!')


def get_dir_name(file_dir):
    base_name = os.path.basename(file_dir)  # 获得地址的文件名
    dir_name = os.path.dirname(file_dir)  # 获得地址的父链接
    return dir_name, base_name


def image_pdf(file_dir):
    dir_name, base_name = get_dir_name(file_dir)
    doc = fitz.Document()
    for img in sorted(glob.glob(file_dir + '\\*'), key=os.path.getmtime):  # 排序获得对象
        img_doc = fitz.Document(img)  # 获得图片对象
        pdf_bytes = img_doc.convertToPDF()  # 获得图片流对象
        img_pdf = fitz.Document("pdf", pdf_bytes)  # 将图片流创建单个的PDF文件
        doc.insertPDF(img_pdf)  # 将单个文件插入到文档
        img_doc.close()
        img_pdf.close()
    doc.save(dir_name + os.sep + base_name + ".pdf")  # 保存文档
    doc.close()
    messagebox.showinfo('提示', '转换成功!')


def pdf_image(pdf_name):
    dir_name, base_name = get_dir_name(pdf_name)
    pdf = fitz.Document(pdf_name)
    for pg in range(0, pdf.pageCount):
        page = pdf[pg]  # 获得每一页的对象
        trans = fitz.Matrix(1.0, 1.0).preRotate(0)
        pm = page.getPixmap(matrix=trans, alpha=False)  # 获得每一页的流对象
        pm.writePNG(FILE[:-4] + os.sep + base_name[:-4] + '_{:0>4d}.png'.format(pg + 1))  # 保存图片
    pdf.close()
    messagebox.showinfo('提示', '转换成功!')


def ui():
    root = tkinter.Tk()
    root.title('PDF和照片互转器')  # 标题
    root.resizable(width=False, height=False)  # 防止大小调整
    canvas = tkinter.Canvas(root, width=450, height=320, highlightthickness=0)  # 创建画布
    photo = tkinter.PhotoImage(file=file_zip_path + os.sep + 'pdf.png')  # 获取背景图片的网络连接
    canvas.create_image(225, 160, image=photo)

    select_dir_button = tkinter.Button(root, text="选择照片文件夹", command=select_dir, bg='yellow')  # 创建按钮
    select_pdf_button = tkinter.Button(root, text="选择PDF文件", command=select_pdf, bg='green')
    click_button = tkinter.Button(root, text="点击执行", command=start, bg='blue')

    select_dir_button.pack()  # 启动按钮
    select_pdf_button.pack()
    click_button.pack()

    canvas.create_window(240, 120, width=100, height=30, window=select_dir_button)  # 将按钮创建到画布
    canvas.create_window(240, 190, width=100, height=30, window=select_pdf_button)
    canvas.create_window(240, 260, width=100, height=30, window=click_button)
    canvas.pack()  # 启动画布
    root.mainloop()  # 主程序循环


if __name__ == '__main__':
    ui()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/5/9 11:50
# @Author: Jtyoui@qq.com

from .SC_ import Lunar, SC  # 阳历转农历
from .CTC_ import CTC  # 农历转阳历
from .BatchCalendar import *  # 批量转换日历



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/5 9:57
# @Author: Jtyoui@qq.com
"""匹配算法"""
from jtyoui.algorithm.SearchAlgorithm import binary_search
from jtyoui.error import NumberValueError


def _verification(number):
    if not isinstance(number, int) or number < 2:
        raise NumberValueError(f'输入的值必须大于1，你输入的值是：{number}')


class FMMA:
    """FMMA(Forward Maximum Matching Algorithms)正向最大匹配算法"""

    def __init__(self, ls, sort=False):
        """正向最大匹配算法、匹配的词典
        :param ls: 词典
        :param sort: 是否要排序
        """
        if sort:
            self.ls = sorted(ls)
        else:
            self.ls = ls

    def cut(self, line, max_length):
        """输入一行字符串，最大按照max_length拆分"""
        _verification(max_length)
        total = []
        while len(line) > 0:
            word = line[:max_length]
            while not binary_search(self.ls, word):
                if len(word) == 1:
                    break
                else:
                    word = word[:-1]
            total.append(word)
            line = line[len(word):]
        return total


class RMMA:
    """RMMA(Reverse Maximum Matching Algorithms)逆向最大匹配算法"""

    def __init__(self, ls, sort=False):
        """逆向最大匹配算法、匹配的词典
        :param ls: 词典
        :param sort: 是否要排序
        """
        if sort:
            self.ls = sorted(ls)
        else:
            self.ls = ls

    def cut(self, line, max_length):
        """输入一行字符串，最大按照max_length拆分"""
        _verification(max_length)
        total = []
        while len(line) > 0:
            word = line[-max_length:]
            while not binary_search(self.ls, word):
                if len(word) == 1:
                    break
                else:
                    word = word[1:]
            total.append(word)
            line = line[:len(line) - len(word)]
        return list(reversed(total))


def _kmp(p):
    i, k, m = 0, -1, len(p)
    p_next = [-1] * m
    while i < m - 1:
        if k == -1 or p[i] == p[k]:
            i, k = i + 1, k + 1
            if p[i] == p[k]:
                p_next[i] = p_next[k]
            else:
                p_next[i] = k
        else:
            k = p_next[k]
    return p_next


def kmp(string, str_):
    """KMP(The Knuth-Morris-Pratt Algorithm)无回溯串匹配算法"""
    j, i = 0, 0
    p_next = _kmp(string)
    n, m = len(string), len(str_)
    while j < n and i < m:
        if i == -1 or string[j] == str_[i]:
            j, i = j + 1, i + 1
        else:
            i = p_next[i]
    if i == m:
        return j - i
    return -1


def max_sub_array(ls: list) -> tuple:
    """求解最大子数组
    :param ls: 数字类列表
    :return:（起始位置，结束位置，最大值）
    """
    min_, max_, sum_ = 0, 0, 0
    return_max = 0, 0, 0
    for index, value in enumerate(ls):
        sum_ += value
        rm = max(sum_, sum_ - ls[index])
        if sum_ < 0:
            sub_max = min_, index - 1, sum_ - ls[index]
            min_, sum_ = index + 1, 0
        elif rm != sum_ and index > 0:
            sub_max = min_, index - 1, rm
        else:
            continue
        return_max = sub_max if return_max[2] < sub_max[2] else return_max
    else:
        return_max = (min_, len(ls) - 1, sum_) if return_max[2] < sum_ else return_max
    return return_max


if __name__ == '__main__':
    r = RMMA(ls=['我们', '野生', '动物园', '在野'], sort=True)
    print(r.cut('我们在野生动物园玩', 3))
    # ['我们', '在', '野生', '动物园', '玩']
    print('-----------------------------------------')
    r = FMMA(ls=['我们', '野生', '动物园', '在野'], sort=True)
    print(r.cut('我们在野生动物园玩', 3))
    # ['我们', '在野', '生', '动物园', '玩']
    print('-----------------------------------------')
    print(kmp('我们在野生动物园玩', '动物园'))
    print('-----------------------------------------')
    print(max_sub_array([5, 4, -12, 1, 3, -1, 4, 1, -6]))



import numpy
from collections import Counter
from keras.preprocessing.sequence import pad_sequences


def load_data(train_path, dev_path=None):
    train = _parse_data(open(train_path, 'rb'))
    word_counts = Counter(row[0].lower() for sample in train for row in sample)
    vocab = [w for w, f in iter(word_counts.items()) if f >= 2]
    chunk_tags = ['O', 'B-a', 'I-a', 'B-b', 'I-b', "B-c", "I-c"]
    train = _process_data(train, vocab, chunk_tags)
    if dev_path:
        dev = _parse_data(open(dev_path, 'rb'))
        dev = _process_data(dev, vocab, chunk_tags)
        return train, dev, (vocab, chunk_tags)
    return train, (vocab, chunk_tags)


def _parse_data(fh, split_text='\n'):
    string = fh.read().decode('utf-8')
    data = [[row.split() for row in sample.split(split_text)] for sample in
            string.strip().split(split_text + split_text)]
    fh.close()
    return data


def _process_data(data, vocab, chunk_tags, max_len=None, one_hot=False):
    if max_len is None:
        max_len = max(len(s) for s in data)
    word2idx = dict((w, i) for i, w in enumerate(vocab))
    x = [[word2idx.get(w[0].lower(), 1) for w in s] for s in data]
    y_chunk = [[chunk_tags.index(w[1]) for w in s] for s in data]
    x = pad_sequences(x, max_len)
    y_chunk = pad_sequences(y_chunk, max_len, value=-1)
    if one_hot:
        y_chunk = numpy.eye(len(chunk_tags), dtype='float32')[y_chunk]
    else:
        y_chunk = numpy.expand_dims(y_chunk, 2)
    return x, y_chunk


def process_data(data, vocab, max_len=100):
    word2idx = dict((w, i) for i, w in enumerate(vocab))
    x = [word2idx.get(w[0].lower(), 1) for w in data]
    length = len(x)
    x = pad_sequences([x], max_len)
    return x, length



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/5/4 16:52
# @Email  : jtyoui@qq.com
# @Software: PyCharm


class TZ:
    """时区名字"""
    Africa_Abidjan = 'Africa/Abidjan'
    Africa_Accra = 'Africa/Accra'
    Africa_Addis_Ababa = 'Africa/Addis_Ababa'
    Africa_Algiers = 'Africa/Algiers'
    Africa_Asmara = 'Africa/Asmara'
    Africa_Bamako = 'Africa/Bamako'
    Africa_Bangui = 'Africa/Bangui'
    Africa_Banjul = 'Africa/Banjul'
    Africa_Bissau = 'Africa/Bissau'
    Africa_Blantyre = 'Africa/Blantyre'
    Africa_Brazzaville = 'Africa/Brazzaville'
    Africa_Bujumbura = 'Africa/Bujumbura'
    Africa_Cairo = 'Africa/Cairo'
    Africa_Casablanca = 'Africa/Casablanca'
    Africa_Ceuta = 'Africa/Ceuta'
    Africa_Conakry = 'Africa/Conakry'
    Africa_Dakar = 'Africa/Dakar'
    Africa_Dar_es_Salaam = 'Africa/Dar_es_Salaam'
    Africa_Djibouti = 'Africa/Djibouti'
    Africa_Douala = 'Africa/Douala'
    Africa_El_Aaiun = 'Africa/El_Aaiun'
    Africa_Freetown = 'Africa/Freetown'
    Africa_Gaborone = 'Africa/Gaborone'
    Africa_Harare = 'Africa/Harare'
    Africa_Johannesburg = 'Africa/Johannesburg'
    Africa_Juba = 'Africa/Juba'
    Africa_Kampala = 'Africa/Kampala'
    Africa_Khartoum = 'Africa/Khartoum'
    Africa_Kigali = 'Africa/Kigali'
    Africa_Kinshasa = 'Africa/Kinshasa'
    Africa_Lagos = 'Africa/Lagos'
    Africa_Libreville = 'Africa/Libreville'
    Africa_Lome = 'Africa/Lome'
    Africa_Luanda = 'Africa/Luanda'
    Africa_Lubumbashi = 'Africa/Lubumbashi'
    Africa_Lusaka = 'Africa/Lusaka'
    Africa_Malabo = 'Africa/Malabo'
    Africa_Maputo = 'Africa/Maputo'
    Africa_Maseru = 'Africa/Maseru'
    Africa_Mbabane = 'Africa/Mbabane'
    Africa_Mogadishu = 'Africa/Mogadishu'
    Africa_Monrovia = 'Africa/Monrovia'
    Africa_Nairobi = 'Africa/Nairobi'
    Africa_Ndjamena = 'Africa/Ndjamena'
    Africa_Niamey = 'Africa/Niamey'
    Africa_Nouakchott = 'Africa/Nouakchott'
    Africa_Ouagadougou = 'Africa/Ouagadougou'
    Africa_Porto_Novo = 'Africa/Porto-Novo'
    Africa_Sao_Tome = 'Africa/Sao_Tome'
    Africa_Tripoli = 'Africa/Tripoli'
    Africa_Tunis = 'Africa/Tunis'
    Africa_Windhoek = 'Africa/Windhoek'
    America_Adak = 'America/Adak'
    America_Anchorage = 'America/Anchorage'
    America_Anguilla = 'America/Anguilla'
    America_Antigua = 'America/Antigua'
    America_Araguaina = 'America/Araguaina'
    America_Argentina_Buenos_Aires = 'America/Argentina/Buenos_Aires'
    America_Argentina_Catamarca = 'America/Argentina/Catamarca'
    America_Argentina_Cordoba = 'America/Argentina/Cordoba'
    America_Argentina_Jujuy = 'America/Argentina/Jujuy'
    America_Argentina_La_Rioja = 'America/Argentina/La_Rioja'
    America_Argentina_Mendoza = 'America/Argentina/Mendoza'
    America_Argentina_Rio_Gallegos = 'America/Argentina/Rio_Gallegos'
    America_Argentina_Salta = 'America/Argentina/Salta'
    America_Argentina_San_Juan = 'America/Argentina/San_Juan'
    America_Argentina_San_Luis = 'America/Argentina/San_Luis'
    America_Argentina_Tucuman = 'America/Argentina/Tucuman'
    America_Argentina_Ushuaia = 'America/Argentina/Ushuaia'
    America_Aruba = 'America/Aruba'
    America_Asuncion = 'America/Asuncion'
    America_Atikokan = 'America/Atikokan'
    America_Bahia = 'America/Bahia'
    America_Bahia_Banderas = 'America/Bahia_Banderas'
    America_Barbados = 'America/Barbados'
    America_Belem = 'America/Belem'
    America_Belize = 'America/Belize'
    America_Blanc_Sablon = 'America/Blanc-Sablon'
    America_Boa_Vista = 'America/Boa_Vista'
    America_Bogota = 'America/Bogota'
    America_Boise = 'America/Boise'
    America_Cambridge_Bay = 'America/Cambridge_Bay'
    America_Campo_Grande = 'America/Campo_Grande'
    America_Cancun = 'America/Cancun'
    America_Caracas = 'America/Caracas'
    America_Cayenne = 'America/Cayenne'
    America_Cayman = 'America/Cayman'
    America_Chicago = 'America/Chicago'
    America_Chihuahua = 'America/Chihuahua'
    America_Costa_Rica = 'America/Costa_Rica'
    America_Creston = 'America/Creston'
    America_Cuiaba = 'America/Cuiaba'
    America_Curacao = 'America/Curacao'
    America_Danmarkshavn = 'America/Danmarkshavn'
    America_Dawson = 'America/Dawson'
    America_Dawson_Creek = 'America/Dawson_Creek'
    America_Denver = 'America/Denver'
    America_Detroit = 'America/Detroit'
    America_Dominica = 'America/Dominica'
    America_Edmonton = 'America/Edmonton'
    America_Eirunepe = 'America/Eirunepe'
    America_El_Salvador = 'America/El_Salvador'
    America_Fort_Nelson = 'America/Fort_Nelson'
    America_Fortaleza = 'America/Fortaleza'
    America_Glace_Bay = 'America/Glace_Bay'
    America_Godthab = 'America/Godthab'
    America_Goose_Bay = 'America/Goose_Bay'
    America_Grand_Turk = 'America/Grand_Turk'
    America_Grenada = 'America/Grenada'
    America_Guadeloupe = 'America/Guadeloupe'
    America_Guatemala = 'America/Guatemala'
    America_Guayaquil = 'America/Guayaquil'
    America_Guyana = 'America/Guyana'
    America_Halifax = 'America/Halifax'
    America_Havana = 'America/Havana'
    America_Hermosillo = 'America/Hermosillo'
    America_Indiana_Indianapolis = 'America/Indiana/Indianapolis'
    America_Indiana_Knox = 'America/Indiana/Knox'
    America_Indiana_Marengo = 'America/Indiana/Marengo'
    America_Indiana_Petersburg = 'America/Indiana/Petersburg'
    America_Indiana_Tell_City = 'America/Indiana/Tell_City'
    America_Indiana_Vevay = 'America/Indiana/Vevay'
    America_Indiana_Vincennes = 'America/Indiana/Vincennes'
    America_Indiana_Winamac = 'America/Indiana/Winamac'
    America_Inuvik = 'America/Inuvik'
    America_Iqaluit = 'America/Iqaluit'
    America_Jamaica = 'America/Jamaica'
    America_Juneau = 'America/Juneau'
    America_Kentucky_Louisville = 'America/Kentucky/Louisville'
    America_Kentucky_Monticello = 'America/Kentucky/Monticello'
    America_Kralendijk = 'America/Kralendijk'
    America_La_Paz = 'America/La_Paz'
    America_Lima = 'America/Lima'
    America_Los_Angeles = 'America/Los_Angeles'
    America_Lower_Princes = 'America/Lower_Princes'
    America_Maceio = 'America/Maceio'
    America_Managua = 'America/Managua'
    America_Manaus = 'America/Manaus'
    America_Marigot = 'America/Marigot'
    America_Martinique = 'America/Martinique'
    America_Matamoros = 'America/Matamoros'
    America_Mazatlan = 'America/Mazatlan'
    America_Menominee = 'America/Menominee'
    America_Merida = 'America/Merida'
    America_Metlakatla = 'America/Metlakatla'
    America_Mexico_City = 'America/Mexico_City'
    America_Miquelon = 'America/Miquelon'
    America_Moncton = 'America/Moncton'
    America_Monterrey = 'America/Monterrey'
    America_Montevideo = 'America/Montevideo'
    America_Montserrat = 'America/Montserrat'
    America_Nassau = 'America/Nassau'
    America_New_York = 'America/New_York'
    America_Nipigon = 'America/Nipigon'
    America_Nome = 'America/Nome'
    America_Noronha = 'America/Noronha'
    America_North_Dakota_Beulah = 'America/North_Dakota/Beulah'
    America_North_Dakota_Center = 'America/North_Dakota/Center'
    America_North_Dakota_New_Salem = 'America/North_Dakota/New_Salem'
    America_Ojinaga = 'America/Ojinaga'
    America_Panama = 'America/Panama'
    America_Pangnirtung = 'America/Pangnirtung'
    America_Paramaribo = 'America/Paramaribo'
    America_Phoenix = 'America/Phoenix'
    America_Port_au_Prince = 'America/Port-au-Prince'
    America_Port_of_Spain = 'America/Port_of_Spain'
    America_Porto_Velho = 'America/Porto_Velho'
    America_Puerto_Rico = 'America/Puerto_Rico'
    America_Punta_Arenas = 'America/Punta_Arenas'
    America_Rainy_River = 'America/Rainy_River'
    America_Rankin_Inlet = 'America/Rankin_Inlet'
    America_Recife = 'America/Recife'
    America_Regina = 'America/Regina'
    America_Resolute = 'America/Resolute'
    America_Rio_Branco = 'America/Rio_Branco'
    America_Santarem = 'America/Santarem'
    America_Santiago = 'America/Santiago'
    America_Santo_Domingo = 'America/Santo_Domingo'
    America_Sao_Paulo = 'America/Sao_Paulo'
    America_Scoresbysund = 'America/Scoresbysund'
    America_Sitka = 'America/Sitka'
    America_St_Barthelemy = 'America/St_Barthelemy'
    America_St_Johns = 'America/St_Johns'
    America_St_Kitts = 'America/St_Kitts'
    America_St_Lucia = 'America/St_Lucia'
    America_St_Thomas = 'America/St_Thomas'
    America_St_Vincent = 'America/St_Vincent'
    America_Swift_Current = 'America/Swift_Current'
    America_Tegucigalpa = 'America/Tegucigalpa'
    America_Thule = 'America/Thule'
    America_Thunder_Bay = 'America/Thunder_Bay'
    America_Tijuana = 'America/Tijuana'
    America_Toronto = 'America/Toronto'
    America_Tortola = 'America/Tortola'
    America_Vancouver = 'America/Vancouver'
    America_Whitehorse = 'America/Whitehorse'
    America_Winnipeg = 'America/Winnipeg'
    America_Yakutat = 'America/Yakutat'
    America_Yellowknife = 'America/Yellowknife'
    Antarctica_Casey = 'Antarctica/Casey'
    Antarctica_Davis = 'Antarctica/Davis'
    Antarctica_DumontDUrville = 'Antarctica/DumontDUrville'
    Antarctica_Macquarie = 'Antarctica/Macquarie'
    Antarctica_Mawson = 'Antarctica/Mawson'
    Antarctica_McMurdo = 'Antarctica/McMurdo'
    Antarctica_Palmer = 'Antarctica/Palmer'
    Antarctica_Rothera = 'Antarctica/Rothera'
    Antarctica_Syowa = 'Antarctica/Syowa'
    Antarctica_Troll = 'Antarctica/Troll'
    Antarctica_Vostok = 'Antarctica/Vostok'
    Arctic_Longyearbyen = 'Arctic/Longyearbyen'
    Asia_Aden = 'Asia/Aden'
    Asia_Almaty = 'Asia/Almaty'
    Asia_Amman = 'Asia/Amman'
    Asia_Anadyr = 'Asia/Anadyr'
    Asia_Aqtau = 'Asia/Aqtau'
    Asia_Aqtobe = 'Asia/Aqtobe'
    Asia_Ashgabat = 'Asia/Ashgabat'
    Asia_Atyrau = 'Asia/Atyrau'
    Asia_Baghdad = 'Asia/Baghdad'
    Asia_Bahrain = 'Asia/Bahrain'
    Asia_Baku = 'Asia/Baku'
    Asia_Bangkok = 'Asia/Bangkok'
    Asia_Barnaul = 'Asia/Barnaul'
    Asia_Beirut = 'Asia/Beirut'
    Asia_Bishkek = 'Asia/Bishkek'
    Asia_Brunei = 'Asia/Brunei'
    Asia_Chita = 'Asia/Chita'
    Asia_Choibalsan = 'Asia/Choibalsan'
    Asia_Colombo = 'Asia/Colombo'
    Asia_Damascus = 'Asia/Damascus'
    Asia_Dhaka = 'Asia/Dhaka'
    Asia_Dili = 'Asia/Dili'
    Asia_Dubai = 'Asia/Dubai'
    Asia_Dushanbe = 'Asia/Dushanbe'
    Asia_Famagusta = 'Asia/Famagusta'
    Asia_Gaza = 'Asia/Gaza'
    Asia_Hebron = 'Asia/Hebron'
    Asia_Ho_Chi_Minh = 'Asia/Ho_Chi_Minh'
    Asia_Hong_Kong = 'Asia/Hong_Kong'
    Asia_Hovd = 'Asia/Hovd'
    Asia_Irkutsk = 'Asia/Irkutsk'
    Asia_Jakarta = 'Asia/Jakarta'
    Asia_Jayapura = 'Asia/Jayapura'
    Asia_Jerusalem = 'Asia/Jerusalem'
    Asia_Kabul = 'Asia/Kabul'
    Asia_Kamchatka = 'Asia/Kamchatka'
    Asia_Karachi = 'Asia/Karachi'
    Asia_Kathmandu = 'Asia/Kathmandu'
    Asia_Khandyga = 'Asia/Khandyga'
    Asia_Kolkata = 'Asia/Kolkata'
    Asia_Krasnoyarsk = 'Asia/Krasnoyarsk'
    Asia_Kuala_Lumpur = 'Asia/Kuala_Lumpur'
    Asia_Kuching = 'Asia/Kuching'
    Asia_Kuwait = 'Asia/Kuwait'
    Asia_Macau = 'Asia/Macau'
    Asia_Magadan = 'Asia/Magadan'
    Asia_Makassar = 'Asia/Makassar'
    Asia_Manila = 'Asia/Manila'
    Asia_Muscat = 'Asia/Muscat'
    Asia_Nicosia = 'Asia/Nicosia'
    Asia_Novokuznetsk = 'Asia/Novokuznetsk'
    Asia_Novosibirsk = 'Asia/Novosibirsk'
    Asia_Omsk = 'Asia/Omsk'
    Asia_Oral = 'Asia/Oral'
    Asia_Phnom_Penh = 'Asia/Phnom_Penh'
    Asia_Pontianak = 'Asia/Pontianak'
    Asia_Pyongyang = 'Asia/Pyongyang'
    Asia_Qatar = 'Asia/Qatar'
    Asia_Qostanay = 'Asia/Qostanay'
    Asia_Qyzylorda = 'Asia/Qyzylorda'
    Asia_Riyadh = 'Asia/Riyadh'
    Asia_Sakhalin = 'Asia/Sakhalin'
    Asia_Samarkand = 'Asia/Samarkand'
    Asia_Seoul = 'Asia/Seoul'
    Asia_Shanghai = 'Asia/Shanghai'
    Asia_BeiJing = 'Asia/Shanghai'
    Asia_Singapore = 'Asia/Singapore'
    Asia_Srednekolymsk = 'Asia/Srednekolymsk'
    Asia_Taipei = 'Asia/Taipei'
    Asia_Tashkent = 'Asia/Tashkent'
    Asia_Tbilisi = 'Asia/Tbilisi'
    Asia_Tehran = 'Asia/Tehran'
    Asia_Thimphu = 'Asia/Thimphu'
    Asia_Tokyo = 'Asia/Tokyo'
    Asia_Tomsk = 'Asia/Tomsk'
    Asia_Ulaanbaatar = 'Asia/Ulaanbaatar'
    Asia_Urumqi = 'Asia/Urumqi'
    Asia_Ust_Nera = 'Asia/Ust-Nera'
    Asia_Vientiane = 'Asia/Vientiane'
    Asia_Vladivostok = 'Asia/Vladivostok'
    Asia_Yakutsk = 'Asia/Yakutsk'
    Asia_Yangon = 'Asia/Yangon'
    Asia_Yekaterinburg = 'Asia/Yekaterinburg'
    Asia_Yerevan = 'Asia/Yerevan'
    Atlantic_Azores = 'Atlantic/Azores'
    Atlantic_Bermuda = 'Atlantic/Bermuda'
    Atlantic_Canary = 'Atlantic/Canary'
    Atlantic_Cape_Verde = 'Atlantic/Cape_Verde'
    Atlantic_Faroe = 'Atlantic/Faroe'
    Atlantic_Madeira = 'Atlantic/Madeira'
    Atlantic_Reykjavik = 'Atlantic/Reykjavik'
    Atlantic_South_Georgia = 'Atlantic/South_Georgia'
    Atlantic_St_Helena = 'Atlantic/St_Helena'
    Atlantic_Stanley = 'Atlantic/Stanley'
    Australia_Adelaide = 'Australia/Adelaide'
    Australia_Brisbane = 'Australia/Brisbane'
    Australia_Broken_Hill = 'Australia/Broken_Hill'
    Australia_Currie = 'Australia/Currie'
    Australia_Darwin = 'Australia/Darwin'
    Australia_Eucla = 'Australia/Eucla'
    Australia_Hobart = 'Australia/Hobart'
    Australia_Lindeman = 'Australia/Lindeman'
    Australia_Lord_Howe = 'Australia/Lord_Howe'
    Australia_Melbourne = 'Australia/Melbourne'
    Australia_Perth = 'Australia/Perth'
    Australia_Sydney = 'Australia/Sydney'
    Canada_Atlantic = 'Canada/Atlantic'
    Canada_Central = 'Canada/Central'
    Canada_Eastern = 'Canada/Eastern'
    Canada_Mountain = 'Canada/Mountain'
    Canada_Newfoundland = 'Canada/Newfoundland'
    Canada_Pacific = 'Canada/Pacific'
    Europe_Amsterdam = 'Europe/Amsterdam'
    Europe_Andorra = 'Europe/Andorra'
    Europe_Astrakhan = 'Europe/Astrakhan'
    Europe_Athens = 'Europe/Athens'
    Europe_Belgrade = 'Europe/Belgrade'
    Europe_Berlin = 'Europe/Berlin'
    Europe_Bratislava = 'Europe/Bratislava'
    Europe_Brussels = 'Europe/Brussels'
    Europe_Bucharest = 'Europe/Bucharest'
    Europe_Budapest = 'Europe/Budapest'
    Europe_Busingen = 'Europe/Busingen'
    Europe_Chisinau = 'Europe/Chisinau'
    Europe_Copenhagen = 'Europe/Copenhagen'
    Europe_Dublin = 'Europe/Dublin'
    Europe_Gibraltar = 'Europe/Gibraltar'
    Europe_Guernsey = 'Europe/Guernsey'
    Europe_Helsinki = 'Europe/Helsinki'
    Europe_Isle_of_Man = 'Europe/Isle_of_Man'
    Europe_Istanbul = 'Europe/Istanbul'
    Europe_Jersey = 'Europe/Jersey'
    Europe_Kaliningrad = 'Europe/Kaliningrad'
    Europe_Kiev = 'Europe/Kiev'
    Europe_Kirov = 'Europe/Kirov'
    Europe_Lisbon = 'Europe/Lisbon'
    Europe_Ljubljana = 'Europe/Ljubljana'
    Europe_London = 'Europe/London'
    Europe_Luxembourg = 'Europe/Luxembourg'
    Europe_Madrid = 'Europe/Madrid'
    Europe_Malta = 'Europe/Malta'
    Europe_Mariehamn = 'Europe/Mariehamn'
    Europe_Minsk = 'Europe/Minsk'
    Europe_Monaco = 'Europe/Monaco'
    Europe_Moscow = 'Europe/Moscow'
    Europe_Oslo = 'Europe/Oslo'
    Europe_Paris = 'Europe/Paris'
    Europe_Podgorica = 'Europe/Podgorica'
    Europe_Prague = 'Europe/Prague'
    Europe_Riga = 'Europe/Riga'
    Europe_Rome = 'Europe/Rome'
    Europe_Samara = 'Europe/Samara'
    Europe_San_Marino = 'Europe/San_Marino'
    Europe_Sarajevo = 'Europe/Sarajevo'
    Europe_Saratov = 'Europe/Saratov'
    Europe_Simferopol = 'Europe/Simferopol'
    Europe_Skopje = 'Europe/Skopje'
    Europe_Sofia = 'Europe/Sofia'
    Europe_Stockholm = 'Europe/Stockholm'
    Europe_Tallinn = 'Europe/Tallinn'
    Europe_Tirane = 'Europe/Tirane'
    Europe_Ulyanovsk = 'Europe/Ulyanovsk'
    Europe_Uzhgorod = 'Europe/Uzhgorod'
    Europe_Vaduz = 'Europe/Vaduz'
    Europe_Vatican = 'Europe/Vatican'
    Europe_Vienna = 'Europe/Vienna'
    Europe_Vilnius = 'Europe/Vilnius'
    Europe_Volgograd = 'Europe/Volgograd'
    Europe_Warsaw = 'Europe/Warsaw'
    Europe_Zagreb = 'Europe/Zagreb'
    Europe_Zaporozhye = 'Europe/Zaporozhye'
    Europe_Zurich = 'Europe/Zurich'
    GMT = 'GMT'
    Indian_Antananarivo = 'Indian/Antananarivo'
    Indian_Chagos = 'Indian/Chagos'
    Indian_Christmas = 'Indian/Christmas'
    Indian_Cocos = 'Indian/Cocos'
    Indian_Comoro = 'Indian/Comoro'
    Indian_Kerguelen = 'Indian/Kerguelen'
    Indian_Mahe = 'Indian/Mahe'
    Indian_Maldives = 'Indian/Maldives'
    Indian_Mauritius = 'Indian/Mauritius'
    Indian_Mayotte = 'Indian/Mayotte'
    Indian_Reunion = 'Indian/Reunion'
    Pacific_Apia = 'Pacific/Apia'
    Pacific_Auckland = 'Pacific/Auckland'
    Pacific_Bougainville = 'Pacific/Bougainville'
    Pacific_Chatham = 'Pacific/Chatham'
    Pacific_Chuuk = 'Pacific/Chuuk'
    Pacific_Easter = 'Pacific/Easter'
    Pacific_Efate = 'Pacific/Efate'
    Pacific_Enderbury = 'Pacific/Enderbury'
    Pacific_Fakaofo = 'Pacific/Fakaofo'
    Pacific_Fiji = 'Pacific/Fiji'
    Pacific_Funafuti = 'Pacific/Funafuti'
    Pacific_Galapagos = 'Pacific/Galapagos'
    Pacific_Gambier = 'Pacific/Gambier'
    Pacific_Guadalcanal = 'Pacific/Guadalcanal'
    Pacific_Guam = 'Pacific/Guam'
    Pacific_Honolulu = 'Pacific/Honolulu'
    Pacific_Kiritimati = 'Pacific/Kiritimati'
    Pacific_Kosrae = 'Pacific/Kosrae'
    Pacific_Kwajalein = 'Pacific/Kwajalein'
    Pacific_Majuro = 'Pacific/Majuro'
    Pacific_Marquesas = 'Pacific/Marquesas'
    Pacific_Midway = 'Pacific/Midway'
    Pacific_Nauru = 'Pacific/Nauru'
    Pacific_Niue = 'Pacific/Niue'
    Pacific_Norfolk = 'Pacific/Norfolk'
    Pacific_Noumea = 'Pacific/Noumea'
    Pacific_Pago_Pago = 'Pacific/Pago_Pago'
    Pacific_Palau = 'Pacific/Palau'
    Pacific_Pitcairn = 'Pacific/Pitcairn'
    Pacific_Pohnpei = 'Pacific/Pohnpei'
    Pacific_Port_Moresby = 'Pacific/Port_Moresby'
    Pacific_Rarotonga = 'Pacific/Rarotonga'
    Pacific_Saipan = 'Pacific/Saipan'
    Pacific_Tahiti = 'Pacific/Tahiti'
    Pacific_Tarawa = 'Pacific/Tarawa'
    Pacific_Tongatapu = 'Pacific/Tongatapu'
    Pacific_Wake = 'Pacific/Wake'
    Pacific_Wallis = 'Pacific/Wallis'
    US_Alaska = 'US/Alaska'
    US_Arizona = 'US/Arizona'
    US_Central = 'US/Central'
    US_Eastern = 'US/Eastern'
    US_Hawaii = 'US/Hawaii'
    US_Mountain = 'US/Mountain'
    US_Pacific = 'US/Pacific'
    UTC = 'utc'


if __name__ == '__main__':
    import datetime
    import pytz

    print(datetime.datetime.now(tz=pytz.timezone(TZ.Asia_BeiJing)))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/7 10:14
# @Author: Jtyoui@qq.com
from jtyoui.ml import sigmoid, error_rate, TEST_LABEL, TRAIN_DATA
import numpy as np

__description__ = """
LR(逻辑回归)算法
"""


def lr_train_bgd(feature: np.array, label: np.array, max_cycle: int, alpha: float) -> np.mat:
    """利用梯度下降法训练逻辑回归模型（LR）
    :param feature:特征
    :param label:标签
    :param max_cycle:最大迭代次数
    :param alpha:学习率
    :return:w的权重
    """
    n = np.shape(feature)[1]  # 特征个数
    w = np.random.rand(n).reshape((n, 1))  # 随机初始化权重
    i = 0
    while i <= max_cycle:
        i += 1
        h = sigmoid(np.dot(feature, w))  # 做点乘，计算sigmoid值
        err = label - h  # 误差
        if i % 100 == 0:
            print(f'error rate: {error_rate(h, label)}')
        w += alpha * np.dot(feature.T, err)  # wi+ 1= wi+ α·d 梯度下降进行权重修正

    return w


if __name__ == '__main__':
    weight = lr_train_bgd(TRAIN_DATA, TEST_LABEL, 2000, 0.01)
    print(sigmoid(np.dot(np.array([[1, 1 / 24, 10 / 60, 32 / 60]], dtype=np.float), weight)))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/11
# @Email : jtyoui@qq.com
# @Software : PyCharm
import turtle

"""
康托三分集
选取一个欧氏长度的直线段，将该线段三等分，去掉中间一段，剩下两段。
将剩下的两段分别再三等分，各去掉中间一段，剩下四段。
将这样的操作继续下去，直到无穷，则可得到一个离散的点集。
点数趋于无穷多，而欧氏长度趋于零。经无限操作，达到极限时所得到的离散点集称之为Cantor集。
"""


class CantorTernarySet:

    def __init__(self, start=-300, end=300, n=6):
        self.start = start  # 初始的起始点
        self.end = end  # 初始的终结点
        self.dot = [[(start, end)]]  # 离散的点集
        self.row(n)

    def row(self, n):
        every_time = algorithm(self.start, self.end)  # 保存每一次分割的点集
        for _ in range(1, n):  # 一共分割多少次
            self.dot.append(every_time)  # 保存每一次分割的点集的集合
            ls = []  # 临时保存每一次点集
            for dot in every_time:  # 遍历每一个点
                s = algorithm(*dot)
                ls.extend(s)
                every_time = ls  # 复制再遍历

    def draw(self):  # 绘画
        pen = turtle.Pen()
        turtle.title('康托三分集')
        pen.width(2)
        for index, dot in enumerate(self.dot):
            for x, y in dot:
                pen.penup()
                pen.goto(x=x, y=100 - index * 10)
                pen.pendown()
                pen.forward(y - x)
        turtle.done()


def algorithm(start, end):
    length = end - start  # 线段的长度
    equal = length / 3
    one_dot = start + equal  # 第一个点的位置
    two_dot = end - equal  # 第二个点的位置
    return [(start, one_dot), (two_dot, end)]


if __name__ == '__main__':
    ct = CantorTernarySet()
    ct.draw()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/21 14:56
# @Author: Jtyoui@qq.com

"""
相关关系
相关关系是一种非确定性的关系，相关系数是研究变量之间线性相关程度的量。
"""

from .Pearson import pearson_coefficient  # 皮尔森相关性系数
from .Spearman import spear_man_coefficient  # 斯皮尔曼相关系数
from .Kendall import kendall_coefficient  # 肯德尔相关性系数



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/9/9 17:16
# @Author: Jtyoui@qq.com
from jtyoui.error import NotLinuxSystemError
import uuid
import socket
import struct
import os


def get_mac_address() -> str:
    """获得本机MAC地址"""
    u = uuid.uuid1().hex[-12:]
    return ':'.join(u[i:i + 2] for i in range(0, len(u), 2))


def get_window_ip():
    """在window下获取ip"""
    name = get_window_name()
    return windowName_get_ip(name)


def get_linux_ip(eth):
    """在Linux下获取IP"""
    assert os.name == 'posix', NotLinuxSystemError('不是Linux系统')
    import fcntl
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    ip = socket.inet_ntoa(fcntl.ioctl(s.fileno(), 0x8915, struct.pack('256s', eth[:15])))
    return ip[20:24]


def auto_get_ip(eth=None):
    """自动获取ip地址,window下eth默认是None。在Linux下，eth对应着不同的网卡"""
    if os.name == 'nt':
        return get_window_ip()
    elif os.name == 'posix':
        return get_linux_ip(eth)
    else:
        raise Exception('暂时支持Linux和window系统')


def windowName_get_ip(window_name):
    """根据window的名字来获取ip地址"""
    return socket.gethostbyname(window_name)


def get_window_name():
    """获取window系统名"""
    return socket.getfqdn(socket.gethostname())


if __name__ == '__main__':
    print(get_window_name())
    print(get_mac_address())
    print(get_window_ip())
    if os.name == 'posix':
        print(get_linux_ip('ens192'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/7/20 13:17
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from jtyoui.web import get
from urllib.request import urlretrieve
import tempfile
import json
import re
import os
import shutil
import time


class BaiDuWenKu:

    def __init__(self, url):
        """爬取百度文库：URL是文库资料地址"""
        self.url = url
        self.id = self.url[29:self.url.find('.html')]

    def load(self, save_path):
        """
        下载资料
        :param save_path: 保存文件地址
        :return: 成功返回True
        """
        dirs = tempfile.mkdtemp()
        print('创建临时文件夹：', dirs)
        types, title, date = self.get_title()
        if types == 'ppt':
            return self._ppt(dirs, save_path, title)
        elif types == 'doc':
            pass
        elif types == 'txt':
            return self._txt()
        elif types == 'pdf':
            return self._pdf(dirs, date)
        return False

    def get_title(self):
        """
        获得资料的标题和类型
        :return:
        """
        data = get(self.url).content.decode('gbk')
        types = re.findall(r'\'docType\': \'\w+\'', data)[0][12:-1]
        title = re.findall(r'\'title\': \'.*\'', data)[0][10:-1]
        return types, title, data

    def _ppt(self, dirs, save_path, title):
        from jtyoui.imagepdf import image_pdf
        """下载带有ppt格式"""
        content_url = "https://wenku.baidu.com/browse/getbcsurl?doc_id=" + self.id + "&pn=1&rn=99999&type=ppt"
        print(content_url)
        content = get(content_url).content.decode('gbk')
        data = json.loads(content)
        start = time.time()
        for size, img in enumerate(data, 1):
            print('\r[下载进度]:%s%.2f%%' % ('>' * int((size * 50 / len(data))), float(size / len(data) * 100)))
            page, zoom = img['page'], img['zoom']
            urlretrieve(zoom, filename=dirs + os.sep + str(page) + '.jpg')
        image_pdf(file_dir=dirs, pdf_address=save_path + os.sep + title)
        shutil.rmtree(dirs)
        end = time.time()
        print('\n下载成功，保存地址：', save_path + os.sep + title + '.pdf', '一共耗时：', end - start, '秒')
        print('删除临时文件夹成功！')
        return True

    def _txt(self):
        url = "https://wenku.baidu.com/api/doc/getdocinfo?callback=cb&doc_id=" + self.id
        print(url)

    def _pdf(self, dirs, html):
        pass


if __name__ == '__main__':
    wk = BaiDuWenKu(url=r'https://wenku.baidu.com/view/f50def7c43323968001c924c.html?sxts=1563610333674')
    wk.load(r'D:')



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/13 15:08
# @Author: Jtyoui@qq.com
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction import DictVectorizer

"""
one-hot编码测试用例
"""
if __name__ == '__main__':
    # .toarray()返回的是ndarray格式，.todense()返回的是matrix格式

    x = [{'city': 'a', 'age': 1}, {'city': 'b', 'age': 2}, {'city': 'c', 'age': 3}]
    d = DictVectorizer()
    print(d.fit_transform(x).toarray())

    corpus = ['UNC played Duke in basketball', 'Duke lost the basketball game']
    count = CountVectorizer()
    print(count.fit_transform(corpus).toarray())



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/19 0019
# @Email : jtyoui@qq.com


def header(request_headers):
    """
    将浏览器中的Request Headers头复制过来.进行改装成字典类型
    :param request_headers:  浏览器中的Request Headers头信息
    :return: 分割Headers头信息的键值对
    """
    headers = dict()
    request_headers = request_headers.strip()
    for head in request_headers.rsplit('\n'):
        name, values = head.split(':', 1)
        headers.setdefault(name.strip(), values.strip())
    return headers


Response_Headers = """
        accept-language: zh-CN,zh;q=0.9,en;q=0.8
        cache-control: max-age=0
        user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36  Chrome/73.0.3683.86
    """

free_header = header(Response_Headers)

if __name__ == '__main__':
    print(header(Response_Headers))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/14 0014
# @Email : jtyoui@qq.com
import requests
from jtyoui.web import random
from jtyoui.error import LibraryNotInstallError
import re

try:
    import itchat  # 安装 pip install itchat
except ModuleNotFoundError:
    raise LibraryNotInstallError("安装 pip install itchat")

url = 'https://m.xunleige.com/'


def movie(name):
    param = dict(searchword=name.encode('gb2312'))
    response = requests.post(url + 'search.asp', data=param, headers={'User-Agent': random()})
    response.encoding = 'GBK'
    data = response.text
    find, = re.findall(pattern=r'<div class="list mb">(.+)</div>', string=data, flags=re.S)
    message = re.findall(pattern='a href="(.+)" title="(.+)" class', string=find)
    return message


@itchat.msg_register(itchat.content.TEXT)  # 获得文本信息
def text_reply(msg):
    message = msg.text
    if message.startswith('@'):
        text = movie(message[1:])
        for u, t in text:
            itchat.send_msg(t + ': ' + url + u, msg['FromUserName'])


def movie_start():
    itchat.auto_login(hotReload=True)
    itchat.run()


if __name__ == '__main__':
    movie_start()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/12 9:59
# @Author: Jtyoui@qq.com
from jtyoui.error import NotFindPipError
import os
import time
import importlib


def pips(module, package=None):
    """安装模块：有时候加载模块和下载模块的名字不一样，需要指定package
    :param module: 加载模块的名字
    :param package: 没有模块下载模块的名字
    """
    package = package.lower() if package else module.lower()
    try:
        m = importlib.import_module(module)
    except ModuleNotFoundError:
        if os.system('pip3.7') in [1, 32512]:  # 1是window退出码，32512是Linux退出码
            if os.system('pip3') in [1, 32512]:
                if os.system('pip') in [1, 32512]:
                    raise NotFindPipError('你的电脑没有安装与pip相关的命令')
                else:
                    p = 'pip'
            else:
                p = 'pip3'
        else:
            p = 'pip3.7'
        os.system(F'{p} install --user {package}')
        time.sleep(2)
        m = importlib.import_module(module)
    return m


if __name__ == '__main__':
    pips('PyMysql')



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/1/31 0031
# @Email : jtyoui@qq.com
# @Software : PyCharm

"""任意进制相互转化"""


def __successive_division(n, x):  # 辗转相除法
    while n:
        yield n % x  # 将余数返回
        n //= x  # 剩下余数


def binary_system(x, base_x, base_y):
    """
    转化进制
    :param x: 字符串非负整数
    :param base_x: 字符串的进制
    :param base_y: 转化的进制
    :return: 被转化的进制
    """
    if base_y <= 1 or base_x <= 1:  # 进制不可能小于1
        raise ValueError('进制不可能小于1')
    if not isinstance(x, (int, str)):
        raise ValueError('x值应该是int类型或者字符串类型')
    if isinstance(x, int):
        if x < 1:
            raise ValueError('x的值不可能小于1')
        x = str(x)
    y = int(x, base_x)  # 将其他进制先转为十进制
    # 在将十进制转为其他进制,并且将大于10的数字用ASCII值来表示,第一个ASCII是97小写的a
    m = map(lambda b: chr(b + 87) if b >= 10 else str(b), __successive_division(y, base_y))
    bs = ''.join(m)[::-1]  # 返回字符串并且反转
    if int(bs, base_y) == y:  # 检验进制是否正确
        return bs
    raise ValueError('验证进制错误!')  # 如果检验失败,返回错误


def gcd(m, n):
    """最大公约数"""
    if n == 0:
        m, n = m, n
    while m != 0:
        m, n = n % m, m
    return n


if __name__ == '__main__':
    print(binary_system(2542, 7, 12))
    print(gcd(97 * 2, 97 * 3))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/18 9:32
# @Author: Jtyoui@qq.com

from jtyoui import parameter_set_length

"""
欧氏距离
欧氏距离是最容易直观理解的距离度量方法，
我们小学、初中和高中接触到的两个点在空间中的距离一般都是指欧氏距离。
"""


@parameter_set_length
def euclidean_distance(coordinate_p, coordinate_q):
    """
    欧氏距离
    :param coordinate_p: p坐标
    :param coordinate_q: q坐标
    :return: 欧氏距离值
    """
    numerator = 0
    for x, y in zip(coordinate_p, coordinate_q):
        numerator += (x - y) ** 2
    return pow(numerator, 0.5)


if __name__ == '__main__':
    print(euclidean_distance((0, 0, 3), (4, 0, 0)))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/2/9 17:56
# @Author: Jtyoui@qq.com
import json
import time
import requests


class CodeRecognition:

    def __init__(self, username, password, app_id, app_key):
        """
        :param username: 用户账号，不是开发者账号
        :param password: 用户密码，不是开发者密码
        :param app_id: 软件ＩＤ，开发者分成必要参数。
        :param app_key: 软件密钥，开发者分成必要参数。
        """
        self.api_url = 'http://api.yundama.com/api.php'
        self.base = {
            'username': username,
            'password': password,
            'appid': str(app_id),
            'appkey': app_key,
        }

    def request(self, fields, files):
        for key, value in files.items():
            files[key] = open(file=value, mode='rb')
        res = requests.post(self.api_url, files=files, data=fields)
        response = res.text
        response = json.loads(response)
        return response

    def upload(self, filename, code_type, timeout):
        data = dict({'method': 'upload', 'codetype': str(code_type), 'timeout': str(timeout)}, **self.base)
        file = dict(file=filename)
        response = self.request(data, file)
        if response:
            return response['ret'] if response['ret'] and response['ret'] < 0 else response['cid']
        else:
            return -9001

    def result(self, cid):
        data = dict({'method': 'result', 'cid': str(cid)}, **self.base)
        response = self.request(data, {})
        return response and response['text'] or ''

    def decode(self, filename, code_type, timeout=60):
        """
        :param filename:  图片文件
        :param code_type: 验证码类型
        :param timeout: 超时时间，秒
        :return: 超时时间（秒），识别结果
        """
        cid = self.upload(filename, code_type, timeout)
        if cid > 0:
            for i in range(timeout):
                r = self.result(cid)
                if r != '':
                    return r
                else:
                    time.sleep(1)
            return ''
        return ''


cr = CodeRecognition('tyoui', 'zw13312324165', 6533, '1928b28e4ce11c541695bbb4510a4c41')

if __name__ == '__main__':
    # 第一个参数是要识别图像的照片。第二参数是图片类型（看README.md文件中的code_type的说明）
    cr = cr.decode('code.png', 3006)
    print(cr)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/14 13:59
# @Author: Jtyoui@qq.com
from jtyoui.web import get
from urllib.parse import quote


def math_tex(tex, file_path=None):
    """根据Tex语言生成数学公式矢量图：关于Tex语法参考：https://blog.csdn.net/qfire/article/details/81382048
    :param tex: Tex语言
    :param file_path: 保存矢量图的地址，后缀名一定是: xxx.svg
    :return: 默认返回SVG数据。有地址保存到地址，返回True
    """
    u = quote(tex)
    name = hash(tex)
    s = get(f'https://math.jianshu.com/math?formula={u}')
    data = s.text
    if not file_path:
        file_path = './' + str(name) + '.svg'
    w = open(file_path, 'w')
    w.write(data)
    w.flush()
    w.close()
    return True


if __name__ == '__main__':
    Tex = r"""\begin{bmatrix}
    0 & \cdots & 0 \\
    \vdots & \ddots & \vdots \\
    0 & \cdots & 0
    \end{bmatrix}"""
    d = math_tex(Tex)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/5/9 17:56
# @Author: Jtyoui@qq.com

import zipfile


def unzip(zip_address, file_name, encoding='UTF-8', sep='\n'):
    """加载zip数据
    :param zip_address:压缩包的地址
    :param file_name:压缩包里面文件的名字
    :param encoding:文件的编码
    :param sep:文件内容换行符
    :return: 压缩包里面的数据：默认编码的UTF-8
    """
    f = zipfile.ZipFile(zip_address)
    fp = f.read(file_name)
    lines = fp.decode(encoding)
    if '\r\n' in lines:
        return lines.split('\r\n')
    elif '\r' in lines:
        return lines.split('\r')
    elif '\n' in lines:
        return lines.split('\n')
    else:
        return lines.split(sep)


if __name__ == '__main__':
    line = unzip('D:\\date.zip', 'date.txt')
    for l in line:
        print(l)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/5 9:37
# @Author: Jtyoui@qq.com
"""搜索算法"""
import bisect


def binary_search(ls, x, sort=False):
    """二分查找算法
    :param ls: 列表。
    :param x: 被查找的数。
    :param sort: 是否要启动排序，False表示不启动排序，默认是不启动。
    :return: 找到返回True，反之亦然
    """
    if sort:
        ls = sorted(ls)
    v = bisect.bisect_left(ls, x)
    if v != len(ls) and ls[v] == x:
        return True
    return False


if __name__ == '__main__':
    s = [1, 2, 3, 4, 5, 6, 10, 7]
    print(binary_search(s, 7, True))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/21 9:52
# @Author: Jtyoui@qq.com

from .distance import *  # 两点之间的距离
from .coefficient import *  # 相关系数
from .analysis import *  # 分析数学
from .dr import *  # 降维
from .clustering import *  # 聚类
from .maths import *  # 数学类



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/11 17:05
# @Author: Jtyoui@qq.com
from jtyoui.web import post, get_js

_sign = """
function a(r, o) {
    for (var t = 0; t < o.length - 2; t += 3) {
        var a = o.charAt(t + 2);
        a = a >= "a" ? a.charCodeAt(0) - 87 : Number(a),
        a = "+" === o.charAt(t + 1) ? r >>> a: r << a,
        r = "+" === o.charAt(t) ? r + a & 4294967295 : r ^ a
    }
    return r
}
var C = null;
var hash = function(r, _gtk) {
    var o = r.length;
    o > 30 && (r = "" + r.substr(0, 10) + r.substr(Math.floor(o / 2) - 5, 10) + r.substr( - 10, 10));
    var t = void 0,
    t = null !== C ? C: (C = _gtk || "") || "";
    for (var e = t.split("."), h = Number(e[0]) || 0, i = Number(e[1]) || 0, d = [], f = 0, g = 0; g < r.length; g++) {
        var m = r.charCodeAt(g);
        128 > m ? d[f++] = m: (2048 > m ? d[f++] = m >> 6 | 192 : (55296 === (64512 & m) && g + 1 < r.length && 56320=== 
        (64512 & r.charCodeAt(g + 1)) ? (m = 65536 + ((1023 & m) << 10) + (1023 & r.charCodeAt(++g)), d[f++] = m >> 18
        |240, d[f++] = m >> 12 & 63 | 128) : d[f++] = m >> 12 | 224, d[f++] = m >> 6 & 63 | 128), d[f++] = 63 & m | 128)
    }
    for (var S = h,
    u = "+-a^+6",
    l = "+-3^+b+-f",
    s = 0; s < d.length; s++) S += d[s],
    S = a(S, u);
    return S = a(S, l),
    S ^= i,
    0 > S && (S = (2147483647 & S) + 2147483648),
    S %= 1e6,
    S.toString() + "." + (S ^ h)
}
"""


def bai_du_translate(word, from_, to_, cookie):
    """百度翻译：需要自己增加cookie
    :param word: 翻译的单词
    :param from_: 该单词是什么语言
    :param to_: 翻译成什么语言
    :param cookie: 百度翻译网址cookie
    :return: 返回json数据
    """
    params = {}
    gtk = '320305.131321201'
    sign = get_js(_sign, 'hash', [word, gtk])
    params['query'] = word
    params['from'] = from_
    params['to'] = to_
    params['sign'] = sign
    params['token'] = 'dbb0f2ac18d5288a0ab1c09e69092827'
    response = post(url='https://fanyi.baidu.com/v2transapi', params=params, cookie=cookie)
    json = response.json()
    return json


class BaiDuLanguage:
    English = 'en'  # 英语
    Chinese = 'zh'  # 中文
    Japan = 'jp'  # 日本
    Arab = 'ara'  # 阿拉伯
    Estonia = 'est'  # 爱沙尼亚
    Bulgarian = 'bul'  # 保加利亚语
    Polish = 'pl'  # 波兰语
    Danish = 'dan'  # 丹麦语
    German = 'de'  # 德语
    Russian = 'ru'  # 俄语
    French = 'fra'  # 法语
    Finnish = 'fin'  # 芬兰语
    Korean = 'kor'  # 韩语
    Dutch = 'nl'  # 荷兰语
    Czech = 'cs'  # 捷克语
    Romanian = 'rom'  # 罗马尼亚语
    Portuguese = 'pt'  # 葡萄牙语
    Swedish = 'swe'  # 瑞典语
    Slovenian = 'slo'  # 斯洛文尼亚语
    Thai = 'th'  # 泰语
    Spanish = 'spa'  # 西班牙语
    Greek = 'el'  # 希腊语
    Hungarian = 'hu'  # 匈牙利语
    Italian = 'it'  # 意大利语
    Vietnamese = 'vie'  # 越南语
    Cantonese = 'yue'  # 粤语


if __name__ == '__main__':
    c = 'BAIDUID=1E9344586B339BDFE673A622D274BAD9:FG=1; BIDUPSID=1E9344586B339BDFE673A622D274BAD9; PSTM=1551668971; REALTIME_TRANS_SWITCH=1; FANYI_WORD_SWITCH=1; HISTORY_SWITCH=1; SOUND_SPD_SWITCH=1; SOUND_PREFER_SWITCH=1; MCITY=-%3A; from_lang_often=%5B%7B%22value%22%3A%22dan%22%2C%22text%22%3A%22%u4E39%u9EA6%u8BED%22%7D%2C%7B%22value%22%3A%22zh%22%2C%22text%22%3A%22%u4E2D%u6587%22%7D%2C%7B%22value%22%3A%22en%22%2C%22text%22%3A%22%u82F1%u8BED%22%7D%5D; BDORZ=B490B5EBF6F3CD402E515D22BCDA1598; locale=zh; Hm_lvt_64ecd82404c51e03dc91cb9e8c025574=1559124247,1559526164,1559625839,1560239569; delPer=0; H_PS_PSSID=1421_21110_29135_29237_28518_29099_28839; PSINO=6; Hm_lpvt_64ecd82404c51e03dc91cb9e8c025574=1560242851; yjs_js_security_passport=412be9bc7e1f3b3ca2628a701b66667a703c1b81_1560242855_js; to_lang_often=%5B%7B%22value%22%3A%22zh%22%2C%22text%22%3A%22%u4E2D%u6587%22%7D%2C%7B%22value%22%3A%22en%22%2C%22text%22%3A%22%u82F1%u8BED%22%7D%2C%7B%22value%22%3A%22jp%22%2C%22text%22%3A%22%u65E5%u8BED%22%7D%5D'
    p = bai_du_translate('hello', BaiDuLanguage.English, BaiDuLanguage.Chinese, c)
    print(p)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/6/1 15:29
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from jtyoui.data.methods import random_lower_char, random_upper_char, random_digits, random_special
from jtyoui.file_zip import sep
import random


class Captcha:
    """验证码生成器
    >>> c = Captcha(300, 60)  # 验证码大小是300*60
    >>> c.format(lower=1, upper=2, digits=4, special=1)  # 有小写字母1个、大写字母2、数字4个、特殊符号1个
    >>> c.make_photo(f'D://')  # 生成的验证码存放在D盘下
    """

    def __init__(self, width=240, height=60):
        self.width = width
        self.height = height
        self.flag = []
        self.number = 4

    @staticmethod
    def color1():
        return random.randint(64, 255), random.randint(64, 255), random.randint(64, 255)

    @staticmethod
    def color2():
        return random.randint(32, 127), random.randint(32, 127), random.randint(32, 127)

    def format(self, lower=4, upper=0, digits=0, special=0):
        """
        验证码格式
        :param lower: 小写字母
        :param upper: 大写字母
        :param digits: 数字
        :param special: 特殊符号
        """
        self.flag.clear()
        self.number = lower + upper + digits + special
        self.flag.extend(random_lower_char(lower))
        self.flag.extend(random_upper_char(upper))
        self.flag.extend(random_digits(digits))
        self.flag.extend(random_special(special))
        random.shuffle(self.flag)

    def make_photo(self, dir_):
        """生成验证码
        :param dir_: 存放验证码照片的文件夹
        """
        from PIL import Image  # 安装pillow： pip install pillow
        from PIL import ImageFont
        from PIL import ImageDraw
        from PIL import ImageFilter
        image = Image.new('RGB', (self.width, self.height), (255, 255, 255))
        font = ImageFont.truetype('arial.ttf', 36)

        draw = ImageDraw.Draw(image)
        for w in range(self.width):
            for h in range(self.height):
                draw.point((w, h), fill=self.color1())
        for index, t in enumerate(self.flag):
            draw.text(((self.width - 10) // self.number * index + 10, 10), t, font=font, fill=self.color2())
        image = image.filter(ImageFilter.BLUR)
        image.save(dir_ + sep + ''.join(self.flag) + '.jpg', 'jpeg')
        return image


if __name__ == '__main__':
    c = Captcha(300, 60)  # 验证码大小是300*60
    for _ in range(10):  # 生成10张验证码
        c.format(lower=1, upper=2, digits=4, special=1)  # 有小写字母1个、大写字母2、数字4个、特殊符号1个
        c.make_photo(r'D://')  # 生成的验证码存放在D盘下



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/29 14:58
# @Author: Jtyoui@qq.com
from urllib.parse import quote
import requests
from jtyoui.web import header
from html.parser import HTMLParser
from jtyoui.data import Languages

# 请求头数据
# https://dict.hjenglish.com/ 更新请求头cookie
_headers = header("""cookie: HJ_CST=1; HJ_CSST_3=1; HJ_SID=58dfee85-f85c-a234-f219-b79dcb7e1e9d; _REF=https://www.baidu.com/link?url%3D1PPFhXl2aDlh33a6R3XE_eUsUW7banuolBKo19dvNXD8eQK8Qf1vvpCORFg1ShyF&wd%3D&eqid%3D812dd4340002b121000000065cc69469; _REG=www.baidu.com|; HJ_SSID_3=ea24368a-e9e2-9b58-9b38-8a90fb70cbae; _SREF_3=https://www.baidu.com/link?url%3D1PPFhXl2aDlh33a6R3XE_eUsUW7banuolBKo19dvNXD8eQK8Qf1vvpCORFg1ShyF&wd%3D&eqid%3D812dd4340002b121000000065cc69469; _SREG_3=www.baidu.com|; HJ_UID=18a93035-bcf4-49bd-a944-1d64e9099ce5; TRACKSITEMAP=3%2C
user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/73.0.3683.103 Safari/537.36""")


class _Html(HTMLParser):

    def __init__(self):
        HTMLParser.__init__(self)
        self._flag = False
        self.string = ''

    def handle_starttag(self, tag, attrs):
        if tag == 'div' and 'simple' in attrs[0]:
            self._flag = True

    def handle_endtag(self, tag):
        if tag == 'div':
            self._flag = False

    def handle_data(self, data):
        if self._flag:
            self.string += data


def translate_to_chinese(name, languages, headers=_headers):
    """将一种语言翻译成中文
    :param name: 单词
    :param languages:语言、及包含：from jtyoui.data import Languages
    :param headers:请求头。更新方法：https://dict.hjenglish.com/
    :return: 中文
    """
    names = quote(name)
    if isinstance(languages, Languages):
        languages = languages.value
    if languages == '英语':
        url = f'https://dict.hjenglish.com/w/{names}'
    elif languages == '日语':
        url = f'https://dict.hjenglish.com/jp/jc/{names}'
    elif languages == '韩语':
        url = f'https://dict.hjenglish.com/kr/{names}'
    elif languages == '法语':
        url = f'https://dict.hjenglish.com/fr/{names}'
    elif languages == '德语':
        url = f'https://dict.hjenglish.com/de/{names}'
    elif languages == '西班牙语':
        url = f'https://dict.hjenglish.com/es/{names}'
    else:
        return ''
    s = requests.get(url, headers=headers)
    date = s.text
    jp = _Html()
    jp.feed(date)
    jp.close()
    s = jp.string
    s = s.replace('\n', '').replace(' ', '')
    if '】' in s:
        index = s.find('】')
        s = s[index + 1:]
    return s


if __name__ == '__main__':
    print(translate_to_chinese('good', Languages.English))  # 英语
    print(translate_to_chinese('アベンジャーズ', Languages.Japanese))  # 日语
    print(translate_to_chinese('안녕하세요', Languages.Korean))  # 韩语
    print(translate_to_chinese('Bonjour', Languages.French))  # 法语
    print(translate_to_chinese('Hallo', Languages.German))  # 德语
    print(translate_to_chinese('Amor', Languages.Spanish))  # 西班牙语
